include <rounded-corner-plate.scad>
include <hexnut.scad>

// A collection of modules for parts to be used in building the
// box.
//
// Things included (eventually):
//
// A drilling guide to be placed over the side panel to locate where to
// drill and cut the holes.
//
// A fan mount to snap into a hole
//
// A mount for an on/off switch
//
// A mount for an A/C connector (computer power supply style).
//
// A plate for ethernet and USB connectors

// The sizes of the box pieces specifically for jiggit's box:
//
jiggit_side_high=82.6;  // need drill guide to be this high
jiggit_side_thick=16.8; // thickness of side walls in box
jiggit_hole_dia=8.33;   // a 21/64 drill bit looks big enough for jigsaw blade
jiggit_plate_wide=55;   // standard width for mounting plates in jiggit
jiggit_plate_high=50;   // standard height for mounting plates in jiggit
jiggit_plate_thick=3;   // standard thickness of mounting plates in jiggit
jiggit_cylinder=(jiggit_side_thick-2)/2; // a little less than half wall thick

// The amount to screw from the bottom side of the top of the box to
// be screwed into a t nut to secure the v-slot on the top of the
// box. So I'll want to make some large "washers" to run the M5 bolts
// through so they can reach this far above the top (or maybe a tad less
// to provide some slop, yet still hold the t-nut securely).
//
jiggit_top_depth=11.75;

// A utility module to produce a 45 degree grid instead of a solid
// plate. NOTE: Openscad apparently needs the number of elements it is
// willing to render bumped way up (in advanced preferences).
//
module grid_plate(xlen,ylen,height,grid_thick=2,gap_thick=2) {
   yl=sqrt(2*ylen*ylen)+4*grid_thick;
   gl=sqrt(2*(gap_thick+grid_thick)*(gap_thick+grid_thick));
   num=floor(1+(ylen+gl+xlen)/gl);
   fx=-((num/2)*gl);
   lx=fx+((num+1)*gl);
   difference() {
      // Make a grid of 45 degree cubes that extend through and out the
      // edges of the desired grid size.
      union() {
         for (i = [fx : gl : lx]) {
            translate([i,0,0])
               rotate([0,0,-45])
                  translate([-grid_thick/2,-yl/2,0])
                     cube([grid_thick,yl,height]);
         }
         for (j = [-fx : -gl : -lx]) {
            translate([j,0,0])
               rotate([0,0,45])
                  translate([-grid_thick/2,-yl/2,0])
                     cube([grid_thick,yl,height]);
         }
      }
      // Subtract the excess from around the edges to produce the
      // desired size grid
      translate([0,0,-1])
         difference() {
            bigx=(xlen*3)+(-fx*2);
            bigy=ylen*3;
            translate([-bigx/2,-bigy/2,0])
               cube([bigx,bigy,height+2]);
            translate([-xlen/2,-ylen/2,-1])
               cube([xlen,ylen,height+4]);
         }
   }
}

// Panel drill guide - print a piece to be placed over a box side panel to
// guide the location of holes to be drilled for the 4 corners of an opening
// to be created by jigsawing lines between the holes.
//
module panel_drill_guide
   (panel_height,  // The height of the side panel the guide fits
    hole_dia,      // The diameter of holes drilled in corners
    hole_width,    // width of hole along panel
    hole_height,   // height of hole in panel
    guide_thick=4) // How thick is guide
{
   fhx=(panel_height-hole_height)/2;
   ydim=hole_width+hole_dia+4+4*guide_thick;
   fhy=(ydim-hole_width)/2;
   xdim=panel_height+(2*guide_thick);
   difference() {
      union() {
         // Use a grid plate for the drill guide
         grid_plate(xdim,ydim,guide_thick,2,5);
         // union edges to the grid
         translate([-xdim/2,-ydim/2,0])
            cube([xdim,guide_thick,guide_thick]);
         translate([-xdim/2,(ydim/2)-guide_thick,0])
            cube([xdim,guide_thick,guide_thick]);
         // edges to slip over the top and bottom of the panel to be drilled
         translate([-xdim/2,-ydim/2,0])
            cube([guide_thick,ydim,2*guide_thick]);
         translate([panel_height+guide_thick-xdim/2,-ydim/2,0])
            cube([guide_thick,ydim,2*guide_thick]);
         // union cylinders around the holes
         for (i = [fhx+guide_thick : hole_height :
                   fhx+guide_thick+hole_height]) {
            for (j = [fhy : hole_width : fhy+hole_width]) {
               translate([i-xdim/2,j-ydim/2,0])
                  union() {
                     // Bigger cylinder for hole to go through
                     cylinder(r=(hole_dia+8)/2, h=guide_thick, $fn=64);
                  }
            }
         }
         // Union cylinder around screw hole
         cylinder(r=6,h=guide_thick,$fn=64);
      }
      union () {
         for (i = [fhx+guide_thick : hole_height :
                   fhx+guide_thick+hole_height]) {
            for (j = [fhy : hole_width : fhy+hole_width]) {
               translate([i-xdim/2,j-ydim/2,-1])
                  union() {
                     // Subtract off the drill holes
                     cylinder(r=hole_dia/2, h=guide_thick+2, $fn=64);
                     // A subtract off notches in the sides of the holes
                     // to allow marking the ends of the lines I want
                     // to follow with the jigsaw.
                     translate([-(hole_dia+4)/2,-2/2,0])
                        cube([hole_dia+4,2,guide_thick+2]);
                     rotate([0,0,90])
                     translate([-(hole_dia+4)/2,-2/2,0])
                        cube([hole_dia+4,2,guide_thick+2]);
                  }
            }
         }
         // Subtract off a hole in the middle for screwing down the
         // drill guide plate if desired to keep it from shifting
         // (the wood with this hole will be going away, so nothing
         // is permanently marked on the box itself).
         translate([0,0,-1])
            cylinder(r=2,h=guide_thick+2,$fn=64);
      }
   }
}

// Subtract this off the plate edge to provide holes for the
// screws (separate module so the subtract can be last after
// unioning in the plate center).
//
module plate_holes(
    hole_dia,      // Same as drill guide value
    hole_width,    // Same as drill guide value
    hole_height,   // Same as drill guide value
    plate_thick,   // How thick is this plate
    wall_high,     // How high to make cylinders that reach into cutout
    do_front=true) // front gets hole for head of bolt, back gets nut trap
{
    // Dimensions of the head of M3 bolts with a bit of slop added
    head_height=3;
    head_dia=5.7;

    // Holes through the piece for M3 bolts
    for (i = [-hole_height/2, hole_height/2]) {
       for (j = [-hole_width/2, hole_width/2]) {
          translate([i,j,-1])
             union () {
                cylinder(r=(3.3)/2, h=wall_high+plate_thick+2,$fn=64);
                // Both the hex nut for the back and the cylinder for the
                // front are too small because the 1st layer gets squished,
                // so make the first 0.4mm of each be slightly larger.
                // Also turn on "External perimiters first" in the slicer
                // settings so it won't try to print the 3mm hole perimeter
                // out in space when it reaches that layer. [NAH, Don't work]
                if (do_front) {
                   union() {
                      // Try making shallow cone at top to see if I
                      // can keep it from trying to print filament
                      // out in space.
                      translate([0,0,head_height+1-0.01])
                         cylinder(r1=(head_dia/2),r2=3.3/2,h=1.2,$fn=64);
                      cylinder(r=head_dia/2,h=head_height+1,$fn=64);
                      cylinder(r=(head_dia+1)/2, h=0.4+1, $fn=64);
                   }
                } else {
                   // Note: Openscad goes crazy without the render here.
                   render() {
                      translate([0,0,m3nut_thick+1])
                         union() {
                            // Try making shallow cone at top to see if I
                            // can keep it from trying to print filament
                            // out in space.
                            translate([0,0,-0.01])
                               cylinder(r1=(head_dia/2),r2=3.3/2,h=1.2,$fn=64);
                            hexnut(m3nut_flat+0.4,m3nut_thick+1);
                            translate([0,0,-(m3nut_thick+1)+(1+0.4)])
                               hexnut(m3nut_flat+1,1+0.4);
                         }
                   }
                }
             }
       }
    }
}

module plate_edge(
    hole_dia,      // Same as drill guide value
    hole_width,    // Same as drill guide value
    hole_height,   // Same as drill guide value
    plate_thick,   // How thick is this plate
    wall_high,     // How high to make cylindersv that reach into cutout
    do_front=true) // front gets hole for head of bolt, back gets nut trap
{
    xlen=hole_height+hole_dia;
    ylen=hole_width+hole_dia;
    cutx=hole_height-0.2;
    cuty=hole_width-0.2;
    // Dimensions of the head of M3 bolts with a bit of slop added
    head_height=3;
    head_dia=5.7;
    difference() {
       union() {
          difference() {
             // basic shape of outer edge of a cover plate
             union() {
                rounded_corner_plate(xlen,ylen,plate_thick,hole_dia/2);
                // Add curved edges, first the corners
                translate([hole_height/2,hole_width/2,plate_thick])
                   rotate_extrude(angle=90,$fn=64)
                      translate([hole_dia/2,0,0])
                         circle(r=plate_thick,$fn=64);
                translate([-hole_height/2,hole_width/2,plate_thick])
                   rotate([0,0,90])
                      rotate_extrude(angle=90,$fn=64)
                         translate([hole_dia/2,0,0])
                            circle(r=plate_thick,$fn=64);
                translate([hole_height/2,-hole_width/2,plate_thick])
                   rotate([0,0,270])
                      rotate_extrude(angle=90,$fn=64)
                         translate([hole_dia/2,0,0])
                            circle(r=plate_thick,$fn=64);
                translate([-hole_height/2,-hole_width/2,plate_thick])
                   rotate([0,0,180])
                      rotate_extrude(angle=90,$fn=64)
                         translate([hole_dia/2,0,0])
                            circle(r=plate_thick,$fn=64);
                // Then the edges
                translate([-(hole_height+0.01)/2,(hole_width+hole_dia)/2,
                           plate_thick])
                   rotate([0,90,0])
                      cylinder(r=plate_thick,h=hole_height+0.01,$fn=64);
                translate([-(hole_height+0.01)/2,-(hole_width+hole_dia)/2,
                           plate_thick])
                   rotate([0,90,0])
                      cylinder(r=plate_thick,h=hole_height+0.01,$fn=64);
                translate([-(hole_height+hole_dia)/2,(hole_width+0.01)/2,
                           plate_thick])
                   rotate([90,0,0])
                      cylinder(r=plate_thick,h=hole_width+0.01,$fn=64);
                translate([(hole_height+hole_dia)/2,(hole_width+0.01)/2,
                           plate_thick])
                   rotate([90,0,0])
                      cylinder(r=plate_thick,h=hole_width+0.01,$fn=64);
             }
             // subtract top half of curved edges off
             translate([-(hole_height+hole_dia+3*plate_thick)/2,
                        -(hole_width+hole_dia+3*plate_thick)/2,
                        plate_thick])
                cube([hole_height+hole_dia+3*plate_thick,
                     hole_width+hole_dia+3*plate_thick,
                     plate_thick+1]);
             // subtract cube from center to provide place to union in
             // whatever custom shape we are making
             translate([-cutx/2,-cuty/2,-1])
                cube([cutx,cuty,plate_thick+2]);
          }
          // cylinders to fit in the holes drilled via the drill guide
          for (i = [-hole_height/2, hole_height/2]) {
             for (j = [-hole_width/2, hole_width/2]) {
                translate([i,j,0])
                   cylinder(r=(hole_dia-0.4)/2, h=wall_high+plate_thick,
                            $fn=64);
             }
          }
       }
       plate_holes(hole_dia, hole_width, hole_height, plate_thick, wall_high,
                   do_front);
    }
}

// One vertical clip to hold the fan in place.
//
module oneclip(clipthick, clipwidth, clipheight) {
   union() {
      cube([clipthick,clipwidth,clipheight]);
      translate([clipthick,0,clipheight-(clipthick/2)])
         rotate([-90,0,0])
            cylinder(h=clipwidth,r=(clipthick/2));
   }
}

// A rectangle with a clip to push fit a fan and a grid to keep fingers out
// of the fan. The default values build a clip for a Noctua 40mm x 10mm fan
// (and perhaps other 40mm fans as well, but I like the silent Noctua ones)
//
module fanclip(
   hole_width,          // Same as drill guide value
   hole_height,         // Same as drill guide value
   hole_dia,            // Same as drill guide value
   wall_high,           // Same as plate_edge value
   plate_thick=3,       // thickness of plate
   fansize=40,          // The size of the fan face
   fandepth=10,         // How thick is the fan itself
   clipthick=3.5,       // How thick to make the clip to hold the fan
   clipwidth=12,        // How wide to make the clip holding the fan
   clipslop=1.5,        // Extra slop at top of fan clip
   fanhole=39) {        // How big to make the hole for the fan to blow thru

   fanthick=fandepth+clipslop;
   clipheight=(fanthick+clipthick);
   fanxy=fansize+(2*clipthick);
   difference() {
      union() {
         difference() {
            translate([-hole_height/2,-hole_width/2,0])
               cube([hole_height,hole_width,plate_thick]);
            translate([clipthick-fanxy/2,clipthick-fanxy/2,(2*plate_thick)/3])
               cube([fansize,fansize,plate_thick]);
            translate([0,0,-1])
               cylinder(h=plate_thick+2, r=(fanhole/2),$fn=64);
         }
         translate([-fanxy/2,-clipwidth/2,(2*plate_thick)/3])
            oneclip(clipthick,clipwidth,clipheight);
         mirror([1,0,0])
            translate([-fanxy/2,-clipwidth/2,(2*plate_thick)/3])
               oneclip(clipthick,clipwidth,clipheight);
         grid_plate(fanxy,fanxy,(2*plate_thick)/3,1.2,5);
         plate_edge(hole_dia,hole_width,hole_height,plate_thick,wall_high,true);
      }
      plate_holes(hole_dia,hole_width,hole_height,plate_thick,wall_high,true);
   }
}


// Subtract this from the power supply cover to leave a hole for mounting the
// A/C cable plug.
//
module plugshape(thick=3) {
   plugx=19.6;
   plugy=28;

   longbase=17;
   shortbase=14.9;

   holedia=3.7;
   holesep=39.4;
   union() {
      intersection() {
         rounded_corner_plate(plugx,plugy,thick,4);

         linear_extrude(height=thick)
            polygon([[-plugx/2, -plugy/2],
                     [(-plugx/2)+shortbase, -plugy/2],
                     [plugx/2, -longbase/2],
                     [plugx/2, longbase/2],
                     [(-plugx/2)+shortbase, plugy/2],
                     [-plugx/2, plugy/2]]);
      }
      translate([0,-holesep/2,0])
         cylinder(r=holedia/2, h=thick, $fn=64);
      translate([0,holesep/2,0])
         cylinder(r=holedia/2, h=thick, $fn=64);
   }
}

// Rectangle with hole for A/C power cord socket (Proper name for socket
// thing is apparently an IEC Power Receptacle).
//
module ac_plug(
   hole_width,          // Same as drill guide value
   hole_height,         // Same as drill guide value
   hole_dia,            // Same as drill guide value
   wall_high,           // Same as plate_edge value
   plate_thick=3) {     // thickness of plate
   difference() {
      union() {
         difference() {
            translate([-hole_height/2,-hole_width/2,0])
               cube([hole_height,hole_width,plate_thick]);
            translate([0,0,-1])
               plugshape(plate_thick+2);
         }
         plate_edge(hole_dia,hole_width,hole_height,plate_thick,wall_high,true);
      }
      plate_holes(hole_dia,hole_width,hole_height,plate_thick,wall_high,true);
   }
}

// Plate with hole to clip in the rocker switch I happen to have (other
// switches would no doubrt require different size holes).
//
module rocker_switch(
   hole_width,          // Same as drill guide value
   hole_height,         // Same as drill guide value
   hole_dia,            // Same as drill guide value
   wall_high,           // Same as plate_edge value
   plate_thick=3,       // thickness of plate
   switch_high=29.5,    // long dimension of rocker switch hole
   switch_wide=10.95) { // narrow dimension of rocker switch hole
   xhole=switch_high;
   yhole=switch_wide;
   difference() {
      union() {
         difference() {
            translate([-(hole_height)/2,-(hole_width)/2,0])
               cube([hole_height,hole_width,plate_thick]);
            translate([-xhole/2,-yhole/2,-1])
               cube([xhole,yhole,plate_thick+2]);
         }
         plate_edge(hole_dia,hole_width,hole_height,plate_thick,wall_high,true);
      }
      plate_holes(hole_dia,hole_width,hole_height,plate_thick,wall_high,true);
   }
}

// Plate with hole to mount a circular emergency stop switch.
//
module stop_switch(
   hole_width,          // Same as drill guide value
   hole_height,         // Same as drill guide value
   hole_dia,            // Same as drill guide value
   wall_high,           // Same as plate_edge value
   plate_thick=3,       // thickness of plate
   switch_dia=22) {     // diameter of switch mounting hole required
   difference() {
      union() {
         difference() {
            translate([-(hole_height)/2,-(hole_width)/2,0])
               cube([hole_height,hole_width,plate_thick]);
            translate([0,0,-1])
               cylinder(r=switch_dia/2,h=plate_thick+2,$fn=64);
         }
         plate_edge(hole_dia,hole_width,hole_height,plate_thick,wall_high,true);
      }
      plate_holes(hole_dia,hole_width,hole_height,plate_thick,wall_high,true);
   }
}

// OK, got the ethernet and usb extension cables in. Time to pull off the
// dimensions:
//
// One the scan of the USB cable, the screws appear to be 613 pixels
// apart and the square USB connector seems to be centered in both
// x and y. That seems real close to 26mm separation.
//
// Getting out the micrometer for ther same USB connector:
//
// The plastic block is 39mm by 15mm (approx)
// The USB connector is 12.2mm wide and 11mm high
// The diameter of the screws is 2.78mm and 28.5mm between outside edges
// Beteeen insidge edges I get 24mm which implies a completely different
// screw diameter, so I assume they are flexing a bit when I measure.
//
// Ah-HA! The amazon page has a pointer to the technical drawing
// which says the USB connector is 12.5mm by 11.5mm and the screws
// are 4-40 separated by 26mm with 3mm diameter diameter holes. These
// are probably the most accurate numbers to use.
//
// Unfortunately no drawing for the ethernet cable.
//
// Screws are 664 pixels apart. That is close to 28mm separation.
//
// The connector is 13.1mm high and 15.9mm wide
//
// The flat side of the connector is 192 pixels from center (most visible)
// The side with the notch for the cable clip is 118 pixels from center
// (that adds up to close to 13.1mm so I might believe the numbers).
//
// The connector block is 21.9mm high and 36.4mm wide
//
// I think I can fit both of these in one plate.
// 

// Model of ethernet connector I can subtract off of a plate to leave
// holes the panel mount ethernet connector I bought will fit. A 2mm
// thickness around the rubbe end connector gives the screws plent of bite
// and brings the ethernet plug almost to the surface of the plate.
// Plugging in an ethernet cable is able to properly click into
// place and be secured.
//
module ethernet_connector(block_thick=10,plate_thick=3) {
   plugx=13.1+0.4;
   plugy=15.9+0.4;
   translate([0,0,block_thick])
   rotate([0,180,0])
   union() {
      // The metal plug for the cable is not centered, it is higher than
      // center, so just hard code the offset.
      translate([-8.128,-plugy/2,0])
         cube([plugx,plugy,block_thick+plate_thick]);
      // Now build the 1/8" "holes" for the screws 28mm apart
      translate([0,28/2,0])
         cylinder(r=3.3/2,h=block_thick+plate_thick,$fn=64);
      translate([0,-28/2,0])
         cylinder(r=3.3/2,h=block_thick+plate_thick,$fn=64);
      // The surrounding plastic block (maybe this needs to shift a bit?)
      translate([-(21.9+0.4)/2,-(36.4+0.4)/2,0])
         cube([21.9+0.4,36.4+0.4,block_thick]);
   }
}

// Model of a USB connector I can subtract off a plate to leave holes
// the panel mount USB extension cable will fit.
//
module usb_connector(block_thick=10,plate_thick=3) {
   plugx=11.5+0.4;
   plugy=12.5+0.4;
   translate([0,0,block_thick])
   rotate([0,180,0])
   union() {
      // The metal connector is centered
      translate([-plugx/2,-plugy/2,0])
         cube([plugx,plugy,block_thick+plate_thick]);
      // Now build the 3mm holes for the screws 26mm apart
      translate([0,26/2,0])
         cylinder(r=3.3/2,h=block_thick+plate_thick,$fn=64);
      translate([0,-26/2,0])
         cylinder(r=3.3/2,h=block_thick+plate_thick,$fn=64);
      // Plus the surrounding hard rubber block
      translate([-15.4/2,-39.4/2,0])
         cube([15.4,39.4,block_thick]);
   }
}

// A module for the USB and ethernet extension cables to mount to
// to bring the data connectors out from the smoothieboard.
//
module data_connectors(
   hole_width,          // Same as drill guide value
   hole_height,         // Same as drill guide value
   hole_dia,            // Same as drill guide value
   wall_high,           // Same as plate_edge value
   plate_thick=3) {     // thickness of plate

   difference() {
      union() {
         difference() {
            translate([-hole_height/2,-hole_width/2,0])
               cube([hole_height,hole_width,plate_thick]);
            union() {
               translate([-9,0,2])
                  rotate([0,0,180]) // clip at bottom of ethernet connector
                     ethernet_connector(plate_thick*2,plate_thick);
               translate([13,0,2]) // usb under ethernet
                 usb_connector(plate_thick*2,plate_thick);
            }
         }
         plate_edge(hole_dia,hole_width,hole_height,plate_thick,wall_high,true);
      }
      plate_holes(hole_dia,hole_width,hole_height,plate_thick,wall_high,true);
   }
}
