// X and Y carriage designs - properties:
//
//    3 mini V-wheels on each carriage
//    wheels rotate along the sides of the X and Y rails
//    belts attach above, but close to, the tops of the carriages
//    the motor on the right drives the top belt
//    the motor on the left drives the bottom belt

// TODO - Create a bracket (or brackets) for mounting the Z motors.
//
// Work out mounting for Z stop (I think it needs to be more adjustable
// than just a t-nut in the rail).

include <gt2belt.scad>
include <rounded-corner-plate.scad>
include <hexnut.scad>
include <rounded-slot.scad>
include <openbuilds-mini-v-wheel.scad>
include <vslot20x20.scad>
include <pulley_and_belt.scad>

// Belt used for drive system

belt_width=6;           // width of GT2 belt
belt_thick=1.38;        // thickness of GT2 belt

pulley_spacer=0.5;       // brass spacers are this thick
geared_idler_thick=8.63; // geared idler pulley thickness
geared_idler_dia=18+4;   // pulley plus slop to clear low profile bolt heads
smooth_idler_thick=8.70; // smooth idler pulley thickness

// The belt_ysep number is a bit arbitrary. It needs to be big enough to be
// able to mount the pulleys and end blocks so they don't interfere with
// one another and small enough to keep the belts between the wheels on the
// X carriage (which seems like a good goal to have).
//
belt_ysep=20;           // pick number that keeps pulleys separated

belt_entry=2;           // length of straight slot for belt entry into blocks

// Wedge used to clamp belt in tapered holes

wedge_angle=5;          // angle of the wedge
wedge_thick=4;          // thickness of the thick end of the wedge
wedge_len=10;           // length of the wedge

// Dimensions related to the mini V wheels

wheel_dia=15.23;      // diameter of wheel
wheel_thick=8.80;     // thickness of wheel
wheel_shim=1;         // The "precision shims" are 1mm thick and go on either
                      // side of the wheel
wheel_spacer=6.35+wheel_shim; // thickness of the spacers used with the wheel
wheel_y_sep=15.95;    // distance from center of rail to hole for wheel
wheel_x_sep=30.32;    // distance from plate center to wheels on either side
wheel_max_dia=7.14;   // diameter of large holes for eccentric spacer
wheel_min_dia=5.02;   // diameter of small holes for bolt

// Dimensions related to the plates used to construct the carriage pieces

plate_thick=4;          // thickness of plates
outer_wall=10;          // extra plate beyond positions of wheel holes

// Dimensions related to the adjustable belt block

top_round=10;           // diameter of cylinder at top of sliding block

// Spacing for t nuts to get them far enough apart for multiple holes to
// have t nuts that don't run into one another

t_nut_spacing=15.5;

// Width of slot in ordinary t-slot extrusions

t_slot_width=6;

// Measurements of the various pulleys with belt thickness taken into account

belt_on_gear=13.7;
belt_on_idler=14.4;
smooth_idler=12.08;

// Low profile M5 bolt info

lp_dia=9+0.2;  // diameter of head of low profile M5 bolt (plus slop)
lp_thick=1.5;  // thickness of low profile bolt head

// Dimentions from 600dpi scan of microswitch:
//
// switch main body: +56+232x478x256
// mounting hole at fulcrum end: +151+389x58x58
// mounting hole at roller end: +375+389x62x61
// roller: +459+23x110x110
// connectors: 100px long
// thickest bit: 6.35mm
// pin1: +94+475x19x95
// pin3: +300+479x19x99
// pin2: +475+479x19x100
//
// 600dpi => 600dots per 25.4mm
// mm per dot => 25.4/600 => 0.04233333333

mm_per_dot=25.4/600;
switch_thick=6.35;

// nema 17 dimensions of interest

nema_screw_sep=31;
nema_disk_height=2;
nema_disk_dia=22;
nema_motor_width=42.3;
nema_body_height=47.35;

motor_width_slop=0.2;

motor_plate_size=nema_motor_width+plate_thick+plate_thick+motor_width_slop;

vslot_width=20;
vslot_width_slop=0.2;

vslot_size=vslot_width+vslot_width_slop;

// E3D v6 dimensions for mount

e3d_big_dia=16;
e3d_small_dia=12;
e3d_top_thick=3.7;
e3d_mid_thick=6;
e3d_bot_thick=3.0;

e3d_offset=8;   // How much extra space in front of X carriage
e3d_up=30;      // How much to raise e3d above lowest possible mount point

// Dimensions for aluminum C channel pieces I'm using on print bed

ch_inside=19.1;
ch_wall=1.7;
ch_deep=13;

// Things I need for the cable chain mounting plate on the y carriage.

y_chain_plate_width=25;
y_chain_plate_grab=8;
carriage_cap_width=49;
cap_extra_mult=0.7;
swiss_hole_dia=3.2;
swiss_hole_sep=2.5*swiss_hole_dia;

// Handy derived formulae

plate_x=2*(wheel_x_sep+outer_wall);
plate_y=2*(wheel_y_sep+outer_wall);
wedge_thin=wedge_thick-(tan(wedge_angle)*wedge_len);

// How far above the plate is the bottom of the lowest belt
above_low=pulley_spacer+(geared_idler_thick-belt_width)/2;

// How far above the plate is the bottom of the highest belt
above_high=pulley_spacer + geared_idler_thick + pulley_spacer +
           (geared_idler_thick-belt_width)/2;

// The right side pulley bracked oriented vertically
module stacked_pulley_bracket() {

   belt_y=(vslot_size/2)+(belt_on_gear/2);
   hole_y=belt_y-(belt_on_idler/2);

   hole_x=plate_thick+(geared_idler_dia/2);

   // Inside height needs to be big enough to hold stacked pulleys
   h1=(3*pulley_spacer)+(2*geared_idler_thick);
   // And also big ehough to have room for 2 t-nuts to hold it to rail
   h2=(2*t_nut_spacing)-(2*plate_thick);
   // Also need to be big enough inside for two t-nut heads to fit
   h3=lp_dia+t_nut_spacing;
   // max of h1 and h2
   h4=(h1 > h2) ? h1 : h2;
   // Pick the max if the two remaining heights.
   inside_height=(h3 > h4) ? h3 : h4;
   outside_height=inside_height+(2*plate_thick);

   bottom_nut=(outside_height-t_nut_spacing)/2;

   difference() {
      union() {
         // vertical piece goes against vertical rail
         cube([plate_thick,vslot_size+plate_thick,outside_height]);
         // with a "fin" on the back to go against side of vertical rail
         translate([-vslot_size, vslot_size, 0])
            cube([vslot_size+plate_thick,plate_thick,outside_height]);
         // and a "fin" in the front for support
         translate([0, vslot_size, 0])
            cube([geared_idler_dia+2+plate_thick,plate_thick,outside_height]);
         // bottom plate has hole positioned for bolt where pulleys with be
         // stacked
         translate([(geared_idler_dia+2+plate_thick)/2,(vslot_size+plate_thick)/2,0])
            rounded_corner_plate(geared_idler_dia+2+plate_thick,vslot_size+plate_thick,
                                 plate_thick,plate_thick/2);
         // top plate like bottom only raised to the top
         translate([(geared_idler_dia+2+plate_thick)/2,(vslot_size+plate_thick)/2,
                     inside_height+plate_thick])
            rounded_corner_plate(geared_idler_dia+2+plate_thick,vslot_size+plate_thick,
                                 plate_thick,plate_thick/2);
      }
      // Subtract off the holes for the t-nuts
      translate([-50,vslot_size/2,bottom_nut])
         rotate([0,90,0])
            cylinder(r=5.2/2,h=100,$fn=64);
      translate([-50,vslot_size/2,bottom_nut+t_nut_spacing])
         rotate([0,90,0])
            cylinder(r=5.2/2,h=100,$fn=64);
      translate([-vslot_size/2,50,bottom_nut])
         rotate([90,00,0])
            cylinder(r=5.2/2,h=100,$fn=64);
      translate([-vslot_size/2,50,bottom_nut+t_nut_spacing])
         rotate([90,00,0])
            cylinder(r=5.2/2,h=100,$fn=64);
      // And the hole for the bolt holding the pulleys
      translate([hole_x, hole_y, -1])
         cylinder(r=5.2/2,h=outside_height+2,$fn=64);
   }
}

// The right side pulley bracket flipped on side to print
module right_stacked_pulley_bracket() {
   rotate([-90,0,0])
      translate([0,-(vslot_size+plate_thick),0])
         stacked_pulley_bracket();
}

// The mirror image of the right bracket for the left bracket.
module left_stacked_pulley_bracket() {
   rotate([90,0,0])
      translate([0,vslot_size+plate_thick,0])
         mirror([0,1,0])
            stacked_pulley_bracket();
}

// Custom corner plate to slide into v-slot verticals and t-slot horizontals
// to hold corners of frame square.
//
module corner_plate(do_left=true) {
   mirror([0,do_left?1:0,0])
   difference() {
      union() {
         // Corner polygon designed for v-slot vertical that starts at the
         // very bottom and t-slot horizontal that butts against side of vertical
         linear_extrude(height=4)
            polygon([[0,0],
                     [20+t_nut_spacing*2,0],
                     [20+t_nut_spacing*2,20],
                     [20,t_nut_spacing*2],
                     [0,t_nut_spacing*2]]);

         // Add in cube I'll subtract the v-slot profile from
         translate([(20-16)/2,0,0])
            cube([16,t_nut_spacing*2,4+1.5]);

         // Add in slightly less than 6mm cube to fit in t-slot
         translate([20+2,(20-6)/2,0])
            #cube([(t_nut_spacing*2)-2,t_slot_width-0.2,4+1.5]);
      }

      // Subtract off v-slot profile from vertical
      translate([10,-1,10+4])
         rotate([-90,0,0])
            vslot20x20(2 + t_nut_spacing*2);

      // Subtract off holes for 5mm screws in vertical slot
      translate([10,t_nut_spacing/2,-1])
         cylinder(r=5.2/2,h=10,$fn=64);
      translate([10,t_nut_spacing + t_nut_spacing/2,-1])
         cylinder(r=5.2/2,h=10,$fn=64);

      // Subtract off holes for 5mm screws in horizontal slot
      translate([20+t_nut_spacing/2,10,-1])
         cylinder(r=5.2/2,h=10,$fn=64);
      translate([t_nut_spacing + 20+t_nut_spacing/2,10,-1])
         cylinder(r=5.2/2,h=10,$fn=64);
   }
}

// Custom corner plate to join two v-slot pieces at right angles (since
// I had extra v-slot, I'm using it for the print bed frame rather than
// ordering more t-slot, hence the need for v-slot profiles in both
// directions).
//
module vv_corner_plate(do_left=true) {
   mirror([0,do_left?1:0,0])
   difference() {
      union() {
         // Corner polygon designed for v-slot vertical that starts at the
         // very bottom and v-slot horizontal that butts against side of vertical
         linear_extrude(height=4)
            polygon([[0,0],
                     [20+t_nut_spacing*2,0],
                     [20+t_nut_spacing*2,20],
                     [20,t_nut_spacing*2],
                     [0,t_nut_spacing*2]]);

         // Add in cube I'll subtract the v-slot profile from
         translate([(20-16)/2,0,0])
            cube([16,t_nut_spacing*2,4+1.5]);

         // Add in another cube at right angles to also subtract v-slot from
         translate([20+2,(20-16)/2,0])
            cube([(t_nut_spacing*2)-2,16,4+1.5]);
      }

      // Subtract off v-slot profile from vertical
      translate([10,-1,10+4])
         rotate([-90,0,0])
            vslot20x20(2 + t_nut_spacing*2);

      // Subtract off v-slot profile from horizontal
      translate([21,10,10+4])
         rotate([0,90,0])
            vslot20x20(2 + t_nut_spacing*2);

      // Subtract off holes for 5mm screws in vertical slot
      translate([10,t_nut_spacing/2,-1])
         cylinder(r=5.2/2,h=10,$fn=64);
      translate([10,t_nut_spacing + t_nut_spacing/2,-1])
         cylinder(r=5.2/2,h=10,$fn=64);

      // Subtract off holes for 5mm screws in horizontal slot
      translate([20+t_nut_spacing/2,10,-1])
         cylinder(r=5.2/2,h=10,$fn=64);
      translate([t_nut_spacing + 20+t_nut_spacing/2,10,-1])
         cylinder(r=5.2/2,h=10,$fn=64);
   }
}

// Shape of the top of the E3D for subtracting from a hot end mount. Add a bit
// of slop so things don't print out too tight to fit. Stick the top and bottom
// out an extra millimeter.
//
module e3d_top() {
   vslop=0.48;
   tot_high = e3d_top_thick+e3d_mid_thick+e3d_bot_thick;
   mid_high=e3d_mid_thick-vslop;
   top_high=e3d_top_thick+(vslop/2);
   bot_high=e3d_bot_thick+(vslop/2);

   union() {
      cylinder(r=(e3d_small_dia+0.2)/2, h=tot_high, $fn=64);
      translate([0,0,-1])
         cylinder(r=(e3d_big_dia+0.2)/2, h=bot_high+1, $fn=64);
      translate([0,0,bot_high+mid_high])
         cylinder(r=(e3d_big_dia+0.2)/2, h=top_high+1, $fn=64);
   }
}

// Mount for e3d hot end
//
module e3d_mount(do_back=true) {
   tot_high = e3d_top_thick+e3d_mid_thick+e3d_bot_thick;
   back_high=2*(plate_thick+10+3) +
             wheel_thick +
             2*wheel_spacer;
   ylen=e3d_big_dia+8+8;

   difference() {
      translate([-(e3d_offset+(e3d_big_dia/2)), -(ylen)/2, 0])
         union() {
            // Position cube for the mount up the back so E3D doesn't
            // stick too far below the carriage
            translate([0,0,e3d_up])
               union() {
                  cube([e3d_big_dia+e3d_offset+4,ylen,tot_high]);
                  // Toss in some cubes to form wire tie supports
                  translate([0,0,tot_high+5]) cube([9,2.5,2.5]);
                  translate([9-2.5,0,tot_high-2]) cube([2.5,2.5,9]);
                  translate([0,ylen-2.5,tot_high+5]) cube([9,2.5,2.5]);
                  translate([9-2.5,ylen-2.5,tot_high-2]) cube([2.5,2.5,9]);
               }
            if (do_back) {
               // The back plate itself.
               cube([3,ylen,back_high]);
               // The riser cube for the bowden tube support
               translate([0,ylen/2-((5+2+2)/2),back_high-4])
                  cube([e3d_offset+e3d_big_dia/2,5+2+2,4]);
               // The cylindrical end of the bowden tube support
               translate([e3d_offset+e3d_big_dia/2,ylen/2,back_high-4])
                  cylinder(r=(5+2+2)/2, h=4, $fn=64);
            }
         }
      // Subtract the shape of the mount from the cube for the mount
      translate([0,0,e3d_up])
         e3d_top();
      // Subtract holes to screw into carriage at bottom
      translate([-(e3d_offset+25),-10,(plate_thick+10+3)-(7+5)])
         rotate([0,90,0])
            cylinder(r=3.2/2,h=50,$fn=64);
      translate([-(e3d_offset+25),10,(plate_thick+10+3)-(7+5)])
         rotate([0,90,0])
            cylinder(r=3.2/2,h=50,$fn=64);
      // Subtract holes for the screws to clamp the front and back
      // E3D mount pieces together
      translate([-(e3d_offset+25),10,e3d_up+e3d_bot_thick+(e3d_mid_thick/2)])
         rotate([0,90,0])
            cylinder(r=3.2/2,h=50,$fn=64);
      translate([-(e3d_offset+25),-10,e3d_up+e3d_bot_thick+(e3d_mid_thick/2)])
         rotate([0,90,0])
            cylinder(r=3.2/2,h=50,$fn=64);
      
      if (do_back) {
         // truncate the front part of the mount when building the back
         translate([0,-25,-1])
            cube([50,50,50]);

         // subtract slots for screwing to the upper part of the carriage
         // (slots rather than holes so the dimensions don't have to come out
         // perfect in the carriage assembly).
         translate([-(e3d_offset+25),-10,back_high-4.5])
            rotate([0,90,0])
               rotate([0,0,90])
                  rounded_slot(1.5,3.2,50);
         translate([-(e3d_offset+25),10,back_high-4.5])
            rotate([0,90,0])
               rotate([0,0,90])
                  rounded_slot(1.5,3.2,50);
         // Subtract cylinder for the bowden tube to run through at top.
         cylinder(r=4.4/2,h=back_high+1,$fn=64);

         // Subtract hole from plate to avoid blocking air flow to
         // cooling fins
         translate([-(e3d_offset+3),0,17])
            rotate([0,-90,0])
               cylinder(r=10,h=10,$fn=64);
      } else {
         // Just the front part of the mounting cube.
         translate([-100,-50,-40])
            cube([100,100,100]);
      }
   }
}

// This cap piece fits over the top of the Z motor and screws into the M3 holes
// as well as the inside t-slot rail. It interlocks with the z_motor_horizontal
// piece (which screws into the top of the rail).
//
module z_motor_cap() {
   rotate([-90,0,0])
   translate([0,-nema_motor_width/2,0])
   difference() {
      union() {
         // Square for top of motor
         translate([-(nema_motor_width+2*plate_thick)/2, -nema_motor_width/2, 0])
            cube([nema_motor_width+2*plate_thick, nema_motor_width, plate_thick]);
         // Walls for side of motor
         translate([(nema_motor_width+motor_width_slop)/2,-nema_motor_width/2,0])
            cube([2*plate_thick, nema_motor_width, nema_body_height+plate_thick]);
         translate([-(2*plate_thick+(nema_motor_width+motor_width_slop)/2),
                    -nema_motor_width/2,0])
            cube([2*plate_thick, nema_motor_width, nema_body_height+plate_thick]);
         // Tabs for holes to attach to inside of rail with t-nuts
         translate([nema_motor_width/2+2*plate_thick-1,
                    nema_motor_width/2-plate_thick,
                    nema_body_height+plate_thick-20])
            difference() {
               cube([1+t_nut_spacing,plate_thick,20]);
               translate([1+(t_nut_spacing/2),5,10])
                  rotate([90,0,0])
                     cylinder(r=5.2/2,h=10,$fn=64);
            }
         translate([-(nema_motor_width/2+2*plate_thick+t_nut_spacing),
                    nema_motor_width/2-plate_thick,
                    nema_body_height+plate_thick-20])
            difference() {
               cube([1+t_nut_spacing,plate_thick,20]);
               translate([t_nut_spacing/2,5,10])
                  rotate([90,0,0])
                     cylinder(r=5.2/2,h=10,$fn=64);
            }
      }
      // subtract off hole size of the disk on top
      translate([0,0,-1])
         cylinder(r=(nema_disk_dia+0.2)/2, h=plate_thick+2, $fn=64);
      // subtract off holes for M3 mounting screws
      for (x = [nema_screw_sep/2, -nema_screw_sep/2]) {
         for (y = [nema_screw_sep/2, -nema_screw_sep/2]) {
            translate([x,y,-1])
               cylinder(r=3.2/2, h=plate_thick+2, $fn=64);
            translate([x,y,-1])
               cylinder(r=6/2, h=plate_thick-1, $fn=64);
         }
      }
      // subtract off slots for interlocking z_motor_horizontal piece
      translate([(nema_motor_width+motor_width_slop)/2+(plate_thick-0.2),
                  -(nema_motor_width+2)/2,
                  nema_body_height+plate_thick-(plate_thick+0.2+20)])
         cube([2*plate_thick,nema_motor_width+2,plate_thick+0.2]);
      translate([-((nema_motor_width+motor_width_slop)/2+(plate_thick-0.2)+
                    2*plate_thick),
                  -(nema_motor_width+2)/2,
                  nema_body_height+plate_thick-(plate_thick+0.2+20)])
         cube([2*plate_thick,nema_motor_width+2,plate_thick+0.2]);
      // subtract off ventilation holes in sides
      translate([-(nema_motor_width+2+4*plate_thick)/2,0,
                 plate_thick+(nema_body_height/2)])
         rotate([0,90,0])
            cylinder(r=(nema_disk_dia+0.2)/2,
                     h=nema_motor_width+2+4*plate_thick, $fn=64);
   }
}

// Horizontal plate that slides into the notches in z_motor_cap and locks
// the motor into place. Has tabs with holes to screw into the top of the
// t-slot rail.
//
module z_motor_horizontal() {
   difference() {
      // Make a rounded corner plate that is too long so I can chop one
      // rounded side.
      translate([0,20-(nema_motor_width+t_nut_spacing+20+20)/2,0])
         rounded_corner_plate(nema_motor_width+2*plate_thick+2*t_nut_spacing,
                              nema_motor_width+t_nut_spacing+20+20,
                              plate_thick,4);
      // Chop off the excess
      translate([-(2+nema_motor_width+2*plate_thick+2*t_nut_spacing)/2,0,-1])
         cube([2+nema_motor_width+2*plate_thick+2*t_nut_spacing,
               50,plate_thick+2]);
      // Subtract off the space for the motor cap piece
      translate([-(nema_motor_width+2*plate_thick)/2,
                 -(nema_motor_width+20), -1])
         cube([nema_motor_width+2*plate_thick,
               1+20+nema_motor_width,plate_thick+2]);
      // Subtract off the holes for nuts to mount to top of rail
      translate([nema_motor_width/2+plate_thick+t_nut_spacing/2,-10,-1])
         cylinder(r=5.2/2,h=plate_thick+2,$fn=64);
      translate([-(nema_motor_width/2+plate_thick+t_nut_spacing/2),-10,-1])
         cylinder(r=5.2/2,h=plate_thick+2,$fn=64);
   }
}

// Nice diagonal support piece for the sides of the motor
module motor_side_support() {
   rotate([90,0,0])
      linear_extrude(height=plate_thick)
         polygon([[plate_thick-motor_plate_size/2,0],
                  [plate_thick-motor_plate_size/2,plate_thick],
                  [(motor_plate_size/2)-plate_thick, t_nut_spacing*2],
                  [(motor_plate_size/2), t_nut_spacing*2],
                  [(motor_plate_size/2), 0]]);
}

// The main motor support itself.
module xy_motor_mount() {
   difference() {
      union() {
         // Main plate the motor face rests against (axis at 0,0)
         translate([plate_thick-motor_plate_size/2, -motor_plate_size/2, 0])
            cube([motor_plate_size-plate_thick, motor_plate_size, plate_thick]);
         // Back plate
         translate([motor_plate_size/2 - plate_thick, -motor_plate_size/2, 0])
            cube([plate_thick,motor_plate_size,t_nut_spacing*2]);
         // Back "fins" to secure to vertical rail
         translate([(nema_motor_width+motor_width_slop)/2,vslot_size/2,0])
            cube([vslot_size,plate_thick,t_nut_spacing*2]);
         translate([(nema_motor_width+motor_width_slop)/2,
                     -(plate_thick + vslot_size/2),0])
            cube([vslot_size,plate_thick,t_nut_spacing*2]);
         translate([0,motor_plate_size/2,0])
            motor_side_support();
         translate([0,plate_thick-motor_plate_size/2,0])
            motor_side_support();
      }
      // Holes for the M3 mounting screws to hold motor down
      translate([nema_screw_sep/2, nema_screw_sep/2, -1])
         cylinder(r=3.2/2, h=plate_thick+2, $fn=64);
      translate([-nema_screw_sep/2, nema_screw_sep/2, -1])
         cylinder(r=3.2/2, h=plate_thick+2, $fn=64);
      translate([nema_screw_sep/2, -nema_screw_sep/2, -1])
         cylinder(r=3.2/2, h=plate_thick+2, $fn=64);
      translate([-nema_screw_sep/2, -nema_screw_sep/2, -1])
         cylinder(r=3.2/2, h=plate_thick+2, $fn=64);
      // Holes on underside for heads of the M3 screws (these will probably
      // need a bit of a cleanup with a 6mm drill bit since they print
      // upside down).
      translate([nema_screw_sep/2, nema_screw_sep/2, -0.3])
         cylinder(r=6/2, h=2, $fn=64);
      translate([-nema_screw_sep/2, nema_screw_sep/2, -0.3])
         cylinder(r=6/2, h=2, $fn=64);
      translate([nema_screw_sep/2, -nema_screw_sep/2, -0.3])
         cylinder(r=6/2, h=2, $fn=64);
      translate([-nema_screw_sep/2, -nema_screw_sep/2, -0.3])
         cylinder(r=6/2, h=2, $fn=64);
      // Depression for the disk around the motor axis to fit
      translate([0,0,plate_thick-(nema_disk_height+0.2)])
         cylinder(r=0.2 + nema_disk_dia/2, h=nema_disk_height+1, $fn=64);
      // Hole for shaft to go through
      translate([0,0,-1])
         cylinder(r=6/2, h=plate_thick+2, $fn=64);
      // Cut out the shape for the vslot rail
      translate([((nema_motor_width+motor_width_slop)/2)-0.5, -vslot_size/2, -1])
         cube([vslot_size+0.5,vslot_size,50]);
      // Punch the holes on the "fins" for T nuts to secure the mount
      translate([(vslot_size/2) + (nema_motor_width+motor_width_slop)/2,
                  0, t_nut_spacing/2])
         rotate([90,0,0])
            translate([0,0,-25])
               cylinder(r=5.2/2, h=50, $fn=64);
      translate([(vslot_size/2) + (nema_motor_width+motor_width_slop)/2,
                  0, t_nut_spacing + t_nut_spacing/2])
         rotate([90,0,0])
            translate([0,0,-25])
               cylinder(r=5.2/2, h=50, $fn=64);
      // Punch a ventilation hole in the side supports
      translate([10, 0, t_nut_spacing-4])
         rotate([90,0,0])
            translate([0,0,-30])
               cylinder(r=12/2, h=60, $fn=64);
   }
}

// wedge_shape creates the basic wedge shaped chunk that will become either
// a hole or the wedge itself. The "extra" parameter can add some slop to make
// it a bit larger for when you are going to subtract it to leave a hole.
//
module wedge_shape(extra=0) {
   translate([0,-(belt_width+extra)/2,0])
      rotate([0,-wedge_angle,0])
         difference() {
            rotate([0,wedge_angle,0])
               cube([wedge_len+extra, belt_width+extra, wedge_thick+extra]);
            translate([-1,-1,-(2+wedge_thick+extra)])
               cube([2+wedge_len+extra, 2+belt_width+extra, 2+wedge_thick+extra]);
         }
}

// A wedge with teeth to go in the hole the belt comes through to wedge it
// in place and keep it from sliding back out. Translated and rotated
// into printable orientation.
//
module toothed_wedge() {
   translate([0,0,wedge_thick])
      rotate([180,0,0])
         rotate([0,-wedge_angle,0])
            difference() {
               rotate([0,wedge_angle,0])
                  wedge_shape();
               translate([-1,-(belt_width+2)/2,-0.01])
                  rotate([-90,0,0])
                     translate([0,-belt_thick/2,0])
                        gt2belt(wedge_len+4, belt_width+2);
            }
}

// Make the block to hold the belt in a fixed position. If do_right is true,
// orient belt to come in from the right side and be fixed on the right half
// of the X carriage (otherwise orient for left side). If do_high is true,
// make the slot be one pulley plus spacer higher to take the high belt.
//
module fixed_belt_block(do_right=true, do_high=false) {
   above_plate=do_high ? above_high : above_low;
   block_height=plate_thick            // provide base to union with plate
                + above_plate          // where above plate does slot start
                + belt_width + 0.2     // slot height plus slop
                + 3;                   // 3mm "roof"
   block_width=2*plate_thick           // thick "walls"
               + wedge_thick;          // plus space for wedge hole
   block_len=wedge_len                 // block holds wedge shaped hole
             + belt_entry;             // plus belt shaped entry hole
   mirror([do_right ? 0 : 1,0,0])
   rotate([0,0,wedge_angle])
      difference() {
         translate([-block_len,-plate_thick,-plate_thick])
            cube([block_len,block_width,block_height]);
         translate([-(block_len+1),0,above_plate])
            cube([block_len+2,belt_thick+0.2,belt_width+0.2]);
         translate([-(wedge_len+0.2+belt_entry),0,above_plate])
            rotate([90,0,0])
               translate([0,(belt_width+0.2)/2,-(wedge_thick+0.2)])
                  wedge_shape(extra=0.2);
      }
}

// Shape of the sliding block that holds belt and is pulled by screw to
// tension belt. Somewhat odd shape so that it can fit a somewhat odd shaped
// hole and have no room to twist or do anything other than slide straight.
// Set extra greater than zero to provide some slop in the profile of the
// shape to subtract it from the fixed part of a block to leave a hole. Set
// extralen to make the shape longer (also for subtracting to leave a hole).
//
module sliding_block_shape(extra=0, extralen=0) {
   above_plate=above_low;
   block_height=above_plate             // bottom part of block
                + belt_width + 0.2      // space for vertical belt plus slop
                + 0.75                  // wall above belt slot
                + m3nut_flat            // nut trap space
                + 0.75;                 // wall above nut trap
   block_width=plate_thick              // 1/2 plate on either side
               + wedge_thick;           // plus wedge in middle
   block_len=wedge_len                  // block holds wedge shaped hole
             + belt_entry;              // plus belt shaped entry hole

   translate([0,(block_width+extra)/2,0])
      union() {
         translate([0,-(block_width+extra)/2,0])
            cube([block_len+extralen,
                  block_width+extra,
                  block_height+extra - ((top_round+extra)/2)]);
         translate([0,0,block_height+extra - ((top_round+extra)/2)])
            rotate([0,90,0])
               cylinder(r=((top_round+extra)/2), h=block_len+extralen, $fn=64);
      }
}

// The actual sliding block piece with the holes cut in it for the belt and
// adjusting screw, flipped 90 degrees to print best. Set do_right to true
// to print block designed to go on right side of carriage, false to print
// block for left side.
//
module sliding_belt_block(do_right=true) {
   above_plate=above_low;
   block_len=wedge_len                  // block holds wedge shaped hole
             + belt_entry;              // plus belt shaped entry hole
   block_width=plate_thick              // 1/2 plate on either side
               + wedge_thick;           // plus wedge in middle
   screw_up=above_plate
            + belt_width + 0.2
            + 0.75
            + (m3nut_flat/2);

   rotate([0,do_right ? -90 : 90,0])
      mirror([do_right ? 0 : 1,0,0])
         difference() {
            // The block positioned so the belt edge is at y==0
            translate([0,-plate_thick/2,0])
               difference() {
                  sliding_block_shape();
                  translate([-1,block_width/2,screw_up])
                     rotate([0,90,0])
                        cylinder(r=3.2/2,h=block_len+2,$fn=64);
                  translate([block_len+1,block_width/2,screw_up])
                     rotate([0,90,0])
                        rotate([0,0,30])
                           hexnut(m3nut_flat+0.2, m3nut_thick+1);
               }
            // Subtract the straight belt shaped entry hole
            translate([-1,0,above_plate])
               cube([block_len+2,belt_thick+0.2,belt_width+0.2]);
            // Subtract the wedge shaped hole in the back
            translate([-0.1,0,above_plate])
               rotate([90,0,0])
                  translate([0,(belt_width+0.2)/2,-(wedge_thick+0.2)])
                     wedge_shape(extra=0.2);

         }
}

module block_slide(do_right=true, do_high=false) {
   above_plate=do_high ? above_high : above_low;
   block_height=above_plate             // bottom part of block
                + belt_width + 0.2      // space for vertical belt plus slop
                + 0.75                  // wall above belt slot
                + m3nut_flat            // nut trap space
                + 0.75;                 // wall above nut trap
   block_width=2*plate_thick            // plate on either side
               + wedge_thick;           // plus wedge in middle
   block_len=wedge_len                  // block holds wedge shaped hole
             + belt_entry               // plus belt shaped entry hole
             + 2                        // additional room to slide
             + 3;                       // thick plate for screw
   extra = 0;
   extralen = 0;
   round = top_round+extra+plate_thick;
   screw_up=above_plate
            + belt_width + 0.2
            + 0.75
            + (m3nut_flat/2);

   translate([0,-(plate_thick+0.1),0])
      mirror([do_right ? 1 : 0,0,0])
         difference() {
            translate([0,(block_width+extra)/2,0])
               union() {
                  translate([0,-(block_width+extra)/2,-plate_thick])
                     cube([block_len+extralen,
                           block_width+extra,
                           plate_thick+block_height+extra - (round/2)]);
                  translate([0,0,block_height+extra - ((top_round+extra)/2)])
                     rotate([0,90,0])
                        cylinder(r=round/2, h=block_len+extralen, $fn=64);
               }
            // Punch hole for sliding block to slide in
            translate([-1,plate_thick/2,do_high?(above_high-above_low):0])
               sliding_block_shape(extra=0.2, extralen=3);
            // Punch hole for M3 adjusting screw
            translate([-1,block_width/2,screw_up])
               rotate([0,90,0])
                  cylinder(r=3.2/2,h=block_len+2,$fn=64);
            // Punch hole for wedge and belt to stick through as sliding block
            // is slid back
            translate([-1,plate_thick-0.2,above_plate-0.2])
               cube([block_len+2,wedge_thick+2*0.2,belt_width+2*0.2]);
         }
}

// A small plate with M3 mounting holes to union into the x_carriage()
//
module mounting_holes() {
   num_holes = floor(((2*wheel_x_sep)-10)/5);
   first_hole = ((2*wheel_x_sep) - (num_holes*5))/2;
   last_hole = first_hole + (5*num_holes);
   translate([-wheel_x_sep,0,0])
      difference() {
         cube([2*wheel_x_sep,plate_thick+3+10,plate_thick]);
         union() {
            for ( i = [first_hole : 5 : last_hole] ) {
               translate([i,7,-1])
                  cylinder(r=3.02/2,h=plate_thick+2,$fn=64);
               translate([i,7+5,-1])
                  cylinder(r=3.02/2,h=plate_thick+2,$fn=64);
            }
         }
      }
}

// Used to build the x_carriage plates. If do_top is true, then the
// additional belt clamps are added to the top of the plate, otherwise
// a simpler bottom plate is built (which will print upside down).
//
module x_carriage(do_top=true) {
   difference() {
      union() {
         // The basic plate structure
         rounded_corner_plate(plate_x,plate_y,plate_thick,outer_wall/2);
         // Add vertical plate with lots of M3 holes for mounting stuff
         // to the carriage
         translate([0,-((plate_y/2)-plate_thick),0])
            rotate([90,0,0])
               mounting_holes();
         if (do_top) {

            // NOTE: Make sure there is enough room to get screws into
            // adjustable blocks once they are printed! May need to make
            // carriage wider so I can leave more room between blocks.
            // (Wider carriage would also be less likely to twist).

            // adjustable block in +x,+y quadrant
            translate([28,belt_ysep/2,plate_thick])
               block_slide(do_right=true, do_high=true);
            // adjustable block in -x,+y quadrant
            translate([-28,belt_ysep/2,plate_thick])
               block_slide(do_right=false, do_high=false);
            // fixed block in +x,-y quadrant
            translate([22,-belt_ysep/2,plate_thick])
               fixed_belt_block(do_right=true, do_high=false);
            // fixed block in -x,-y quadrant
            translate([-22,-belt_ysep/2,plate_thick])
               fixed_belt_block(do_right=false, do_high=true);
         }
      }
      // Subtract off hole for center wheel (uses eccentric spacer)
      translate([0,wheel_y_sep,-1])
         cylinder(r=wheel_max_dia/2,h=plate_thick+2,$fn=64);
      // Subtract off holes for other wheels (normal spacers)
      translate([wheel_x_sep,-wheel_y_sep,-1])
         cylinder(r=wheel_min_dia/2,h=plate_thick+2,$fn=64);
      translate([-wheel_x_sep,-wheel_y_sep,-1])
         cylinder(r=wheel_min_dia/2,h=plate_thick+2,$fn=64);
   }
}

// A cap to go on top of the Y carriage and be screwed down tightly by
// the bolts for the belt pulleys which will extend through the cap.
// There are sturdy supports which fit into holes in the Y carriage
// so everything is locked down tight.
//
module carriage_cap(do_right=true) {

   cw=carriage_cap_width;
   low_belt=plate_thick + above_low;
   high_belt=low_belt+(above_high-above_low);
   support_height=3*pulley_spacer+2*geared_idler_thick+plate_thick;
   support_dia=9;
   cap_extra=(plate_y-carriage_cap_width)/2;

   smooth_y=(belt_on_gear/2)      // out from center of drive gear
            + (smooth_idler/2);   // in to center of smooth idler pulley
   smooth_x=(belt_ysep/2)         // position belt to join X carriage
            - (smooth_idler/2);

   pulley_y=(belt_on_gear/2)      // out from center of drive gear
            -(belt_on_idler/2);   // back in to center of geared idler
   pulley_x=(belt_ysep/2)         // position belt to join X carriage
            - (belt_on_idler/2);

   rotate([0,180,0])
   translate([0,0,-(plate_thick + support_height)])
   mirror([do_right ? 0 : 1,0,0])
   difference() {
      union() {
         for (x = [-20, 20]) {
            for (y = [-20,0,20]) {
               translate([x,y,plate_thick])
                  union() {
                     cylinder(r=support_dia/2,h=support_height,$fn=64);
                     translate([0,0,-plate_thick])
                        cylinder(r=5/2,h=support_height,$fn=64);
                  }
            }
         }
         translate([0,0,support_height])
            rounded_corner_plate(cw,cw,plate_thick,support_dia/2);
      }
      translate([-smooth_x,-smooth_y,0])
         cylinder(r=5/2,h=support_height*2,$fn=64);
      translate([pulley_x,pulley_y,0])
         cylinder(r=5/2,h=support_height*2,$fn=64);
      // Subtract off bolt hole for right side cable chain mounting plate
      translate([(do_right?(10+3):100),
                 ((plate_y/2)-(plate_thick+(cap_extra*cap_extra_mult))),
                 -1])
         cylinder(r=(swiss_hole_dia/2),h=support_height*2,$fn=64);
   }
}

// Used to build the Y carriage platform. Holes for the mini wheels as
// well as the pulleys for the belts, plus an extension platform to hold
// the end of the X rail at right angles to the Y rail.
//
module y_carriage(do_top=true, do_right=true) {

   plate_sep = wheel_thick + 2*wheel_spacer;
   extra_z = (plate_sep - 20)/2;
   xten=12;             // center of carriage to end of X rail

   smooth_y=(belt_on_gear/2)      // out from center of drive gear
            + (smooth_idler/2);   // in to center of smooth idler pulley
   smooth_x=(belt_ysep/2)         // position belt to join X carriage
            - (smooth_idler/2);

   pulley_y=(belt_on_gear/2)      // out from center of drive gear
            -(belt_on_idler/2);   // back in to center of geared idler
   pulley_x=(belt_ysep/2)         // position belt to join X carriage
            - (belt_on_idler/2);

   low_belt=plate_thick + above_low;
   high_belt=plate_thick + above_high;
   cap_wide=carriage_cap_width;
   cap_extra=(plate_y-cap_wide)/2;
   cxm=cap_extra_mult;

   rotate([0,do_top ? 180 : 0,0])
   translate([0,0,do_top ? -plate_thick : 0])
   mirror([do_right ? 0 : 1,0,0])
   difference() {
      union() {
         // The basic plate for the carriage
         rounded_corner_plate(plate_x,plate_y,plate_thick,outer_wall/2);
         // Raised surface for X rail to sit on to bring to same level
         // as Y rail
         translate([-(20+2*plate_thick)/2, -(xten + 2*t_nut_spacing),
                    do_top ? -extra_z : 0])
            cube([20+2*plate_thick, 2*t_nut_spacing, plate_thick+extra_z]);
         // Plus vertical for connecting to side of X rail
         translate([do_top ? 10 : -(10 + plate_thick),
                    -(xten + 2*t_nut_spacing),
                    do_top ? -(extra_z+20) : 0])
            cube([plate_thick, 2*t_nut_spacing, plate_thick+extra_z+20]);
         if (! do_top) {
            // Add a short rail for a part holding the limit switch to
            // slide against on the back half of the bottom carriage.
            translate([-16,-(plate_y/2),plate_thick-0.1])
               cube([3,10,3.1]);
         }
      }
      // Subtract off hole for center wheel (uses eccentric spacer)
      translate([0,wheel_y_sep,-1])
         cylinder(r=wheel_max_dia/2,h=plate_thick+2,$fn=64);
      // Subtract off holes for other wheels (normal spacers)
      translate([wheel_x_sep,-wheel_y_sep,-1])
         cylinder(r=wheel_min_dia/2,h=plate_thick+2,$fn=64);
      translate([-wheel_x_sep,-wheel_y_sep,-1])
         cylinder(r=wheel_min_dia/2,h=plate_thick+2,$fn=64);
      // Subtract off holes for t nuts to secure X rail
      translate([0,-(xten + t_nut_spacing/2),-10])
         cylinder(r=wheel_min_dia/2,h=20,$fn=64);
      translate([0,-(xten + t_nut_spacing + t_nut_spacing/2),-10])
         cylinder(r=wheel_min_dia/2,h=20,$fn=64);
      translate([0,-(xten + t_nut_spacing/2),
                  do_top ? -(extra_z+10) : (plate_thick+extra_z+10)])
         rotate([0,do_top ? 90 : -90,0])
            cylinder(r=wheel_min_dia/2,h=20,$fn=64);
      translate([0,-(xten + t_nut_spacing + t_nut_spacing/2),
                  do_top ? -(extra_z+10) : (plate_thick+extra_z+10)])
         rotate([0,do_top ? 90 : -90,0])
            cylinder(r=wheel_min_dia/2,h=20,$fn=64);
      // Holes for the pulleys only needed in the top carriage,
      // hole for limit switch slot only needed in bottom
      if (do_top) {
         // Smooth idler hole with recess for low profile bolt head
         translate([-smooth_x,-smooth_y,-(50-lp_thick)])
            cylinder(r=lp_dia/2,h=50,$fn=64);
         translate([-smooth_x,-smooth_y,-plate_thick])
            cylinder(r=wheel_min_dia/2,h=50,$fn=64);
         // Geared idler hole with recess for low profile bolt head
         translate([pulley_x,pulley_y,-(50-lp_thick)])
            cylinder(r=lp_dia/2,h=50,$fn=64);
         translate([pulley_x,pulley_y,-plate_thick])
            cylinder(r=wheel_min_dia/2,h=50,$fn=64);

         // Holes for carriage_cap() to mate with
         for (x = [-20, 20]) {
            for (y = [-20,0,20]) {
               translate([x,y,-0.1])
                  cylinder(r=5.2/2,h=plate_thick+2,$fn=64);
            }
         }
      } else {
         // Make slot for screw in limit switch holder to slide in
         translate([-20,5-(plate_y/2),-0.1])
            rounded_slot(3,3.2,plate_thick+0.2);
      }
      if (do_right) {
         // On right side, subtract off holes for bolting on the
         // cable chain mounting plate.
         translate([10+3,(plate_y/2)-(plate_thick+(cap_extra*cxm)),-1])
            cylinder(r=(swiss_hole_dia/2), h=plate_thick*2, $fn=64);
         translate([10+y_chain_plate_width-3,
                    (plate_y/2)-(plate_thick+(cap_extra*cxm)),-1])
            cylinder(r=(swiss_hole_dia/2), h=plate_thick*2, $fn=64);
      }
   }
}

// A little slide to hold a limit switch to be attached to the slot in the
// bottom Y carriage.
//
module switch_holder(do_right=true) {
   switch_thick=6.35;
   extra_thick=(switch_thick-plate_thick)/2;
   holder_thick=2.7;
   tot_thick=extra_thick+holder_thick;
   hole_y=20;
   hole_x=10;
   holder_len=2*t_nut_spacing - 3;

   mirror([0,do_right?0:1,0])
   translate([0,0,tot_thick])
   rotate([0,180,0])
   difference() {
      union() {
         // Cube that slides on Y carriage
         cube([8,holder_len-11-6,tot_thick]);
         translate([0,0,extra_thick])
            cube([8,holder_len,holder_thick]);
         // Cube that holds the switch
         translate([8-0.1,holder_len-11,extra_thick])
            cube([25.1-8,11,holder_thick]);
      }
      // Hole for screw to hold to Y carriage
      translate([4,7,-1])
         cylinder(r=3.2/2,h=10,$fn=64);
      // Holes for slots for switch to slide in
      translate([hole_x,hole_y,-10])
            rotate([0,0,90])
               rounded_slot(2.5, (65*mm_per_dot), 50);
      translate([hole_x+(375-151)*mm_per_dot,hole_y,-10])
            rotate([0,0,90])
               rounded_slot(2.5, (65*mm_per_dot), 50);
   }
}

// A holder for a switch that goes on the Y rail to limit Y motion.
// Can also be used as the Z limit switch on one of the vertical rails.
// Use elen to make a longer than normal piece for the z-stop.
//
module ystop(do_back=true, do_preview=false, elen=0) {
   mirror([0,do_back?0:1,0])
   difference() {
      union() {
         cube([3,20,elen+23.1]);
         translate([0,0,elen+23+2])
            cube([4.5,20,16]);
         translate([0,0,elen+23])
            cube([3,20,20]);
         if (do_preview) {
            translate([3,0,0])
               rotate([0,0,180])
                  rotate([0,-90,0])
                     microswitch();
         }
      }

      translate([10+3,21,elen+23+10])
         rotate([90,0,0])
            vslot20x20(22);

      translate([-10,10,elen+23+10])
         rotate([0,90,0])
            cylinder(r=(5.2/2), h=20, $fn=64);

      translate([-3,7.8,5])
         rotate([0,90,0])
            rotate([0,0,90])
               rounded_slot(3,2.6,10);

      translate([-3,7.8,15])
         rotate([0,90,0])
            rotate([0,0,90])
               rounded_slot(3,2.6,10);
   }
}

// Stiffner for smoothieboard mount
module stiffner() {
   difference() {

      // angled plate wide near rail, getting narrower at far end

      linear_extrude(height=plate_thick)
         polygon([[plate_thick,-plate_thick],
                  [-20,-plate_thick],
                  [-20,0],
                  [-10,135.90],
                  [plate_thick,135.90]]);

      // Subtract off handy array of M3 size holes for mounting
      // random stuff to side (hold cover on, mount extra 12V headers, etc).
      for (i = [0 : 8]) {
         translate([-(plate_thick+2),3+15*i,-1])
            cylinder(r=3.2/2, h=plate_thick+2, $fn=32);
      }

      // Punch a bunch of additional strain relief slots in the vertical
      // walls.

      for (s = [0 : 8]) {
         translate([((10/8)*s)/2-(1.5+3+plate_thick),s*15+3+15/2,-1])
            rotate([0,0,90])
               rounded_slot(14-((10/8)*s),3,plate_thick+2);
      }
   }
}

// Wall at half way point where I split the board into two pieces.
//
module smoothie_split() {
   xdim=111.20+2*plate_thick;
   difference() {
      cube([xdim,5,plate_thick+14]);
      // Subtract off holes for M3 nut trap and screws
      // to use to secure two halfs of mount together
      translate([xdim/4,1-m3nut_thick,7])
          rotate([90,0,0])
             rotate([0,0,30])
                union() {
                   hexnut(m3nut_flat+0.2, m3nut_thick);
                   translate([0,0,-6])
                      cylinder(r=3.2/2,h=7,$fn=32);
                   translate([0,0,2-(m3nut_thick+7+5)])
                      cylinder(r=6.5/2, h=7, $fn=32);
                }
      translate([3*xdim/4,1-m3nut_thick,7])
          rotate([90,0,0])
             rotate([0,0,30])
                union() {
                   hexnut(m3nut_flat+0.2, m3nut_thick);
                   translate([0,0,-6])
                      cylinder(r=3.2/2,h=7,$fn=32);
                   translate([0,0,2-(m3nut_thick+7+5)])
                      cylinder(r=6.5/2, h=7, $fn=32);
                }
      // Subtract off slots for strain relief
      for (s = [0:6]) {
         if ((s != 1) && (s != 3) && (s != 5)) {
            translate([(s+1)*(xdim/8),6,2+8/2+3/2])
               rotate([90,0,0])
                  rounded_slot(8,3,5+2);
         }
      }
   }
}

// Mount for smoothieboard. Print in two pieces to fit in my solidoodle 2.
// If do_near is true, print half nearest the rail, if false, half farthest
// from rail.
//
// Still having trouble getting this to print without curling. Add some
// grooves along the top for strain relief. Then add slots in the verticals
// for more strain relief. The check the bed level. Then finally get both
// pieces to print without curling up by using only 2 perimeters and zero
// top and bottom layers (just honeycomb infill top to bottom at 20%).
//
// I'll wait till Jiggit is working to design and print a cover :-).
//
module smoothie(do_near=true) {
   ydim = 135.90+plate_thick+20;
   // Rotate to printable position
   rotate([0,180,0])
   rotate([0,0,do_near ? 90 : 270])
   difference() {
      union() {
         // Position mounting plate so the board itself lives in positive (x,y)
         // quadrant and the various edges and tabs for support extend out
         // from there.
         translate([-plate_thick,-(plate_thick+20),0])
            cube([111.20+2*plate_thick,ydim,plate_thick]);

         // Make vertical to provide mount on inside of front rail
         translate([-plate_thick,-plate_thick,-20])
            cube([111.20+2*plate_thick,plate_thick,20+plate_thick]);

         // Make vertical to provide place where I can split two halves.
         translate([-plate_thick,(ydim/2)-(plate_thick+20+(5/2)),-14])
            render()
               smoothie_split();

         // Add stiffners
         rotate([0,-90,0])
            render() stiffner();
         translate([111.20+plate_thick,0,0])
            rotate([0,-90,0])
               render() stiffner();
         translate([(-plate_thick/2)+(111.20+2*plate_thick)/2,0,0])
            rotate([0,-90,0])
               render() stiffner();
      }

      // Subtract the holes for the board mounting screws (I'll need to
      // find or print some standoffs to hold board up over surface).
      translate([4.2,3.5,-1])
         cylinder(r=1.8, h=plate_thick+2, $fn=64);
      translate([3.6,102.5,-1])
         cylinder(r=1.8, h=plate_thick+2, $fn=64);
      translate([101.6,103,-1])
         cylinder(r=1.8, h=plate_thick+2, $fn=64);
      translate([96.7,10,-1])
         cylinder(r=1.8, h=plate_thick+2, $fn=64);

      // Subtract the holes for the screws that mount to top of front rail
      translate([-plate_thick+(111.20+2*plate_thick)/4,-(plate_thick+10),-1])
         cylinder(r=5.2/2, h=plate_thick+2, $fn=64);
      translate([-plate_thick+3*((111.20+2*plate_thick)/4),-(plate_thick+10),-1])
         cylinder(r=5.2/2, h=plate_thick+2, $fn=64);

      // Subtract the holes for the screws on the inside rail
      translate([-plate_thick+(111.20+2*plate_thick)/4,1,-10])
         rotate([90,0,0])
            cylinder(r=5.2/2, h=plate_thick+2, $fn=64);
      translate([-plate_thick+3*((111.20+2*plate_thick)/4),1,-10])
         rotate([90,0,0])
            cylinder(r=5.2/2, h=plate_thick+2, $fn=64);

      // Subtract off small grooves in the top surface (which will be
      // the bottom when printed

      for (j = [0 : 4]) {
         translate([-(plate_thick+2),j*(139.5/5),plate_thick])
            rotate([0,90,0])
               cylinder(r=1/2, h=500, $fn=16);
         translate([j*(111.2/4),0,plate_thick])
            rotate([-90,0,0])
               cylinder(r=1/2, h=500, $fn=16);
      }

      if (do_near) {
         // Subtract off far half
         translate([-(1+plate_thick),(ydim/2)-(20+plate_thick),-21])
            cube([2+111.20+2*plate_thick, ydim, 2+20+plate_thick]);
      } else {
         // Subtract off near half
         translate([-(1+plate_thick),-(ydim/2+20+plate_thick),-21])
            cube([2+111.20+2*plate_thick, ydim, 2+20+plate_thick]);
      }
   }
}

// Form a C channel shape to subtract off leaving hole to insert channel.
//
module channel(ins, wall, deep, len) {
   union() {
      difference() {
         cube([len, ins+2*wall, deep+wall]);
         translate([-1, wall, wall])
            cube([len+2, ins, deep+1]);
      }

      // For some reason there is a tiny ridge in my C channel down the
      // middle of the bottom, so include that in the model.
      translate([0,(ins+2*wall)/2, wall])
         rotate([0,90,0])
            cylinder(r=0.5, h=len, $fn=32);

      // Corners are hard to print perfectly square, so make the model
      // have some rounded corners that stick out a bit to leave room.
      rotate([0,90,0])
         cylinder(r=0.7, h=len, $fn=32);
      translate([0,ins+2*wall,0])
         rotate([0,90,0])
            cylinder(r=0.7, h=len, $fn=32);

      // Round the inside corners as well.
      difference() {
         cube([len, wall+0.7, wall+0.7]);
         translate([-1,wall+0.7,wall+0.7])
            rotate([0,90,0])
               cylinder(r=0.7, h=len+2, $fn=32);
      }
      difference() {
         translate([0,(ins+2*wall)-(wall+0.7),0])
            cube([len, wall+0.7, wall+0.7]);
         translate([-1,ins+2*wall-(wall+0.7),wall+0.7])
            rotate([0,90,0])
               cylinder(r=0.7, h=len+2, $fn=32);
      }
   }
}

// Bracket to hold a C channel under the print bed where the leveling
// screws can be attached.
//
module bedbracket() {
   translate([0,0,20+plate_thick])
   rotate([0,90,0])
   difference() {
      union() {
         // Cube around end of C channel
         translate([0,-plate_thick,-plate_thick])
            cube([20+plate_thick,ch_inside+2*ch_wall+2*plate_thick,
                  ch_deep+ch_wall+plate_thick]);

         // Tabs for holes to secure with t-nuts
         translate([0,-(plate_thick+t_nut_spacing),
                    (ch_deep+ch_wall)-plate_thick])
            cube([20+plate_thick,
                  ch_inside+2*ch_wall+2*plate_thick+2*t_nut_spacing,
                  plate_thick]);

         // Cube for right angle join to rail
         translate([20, -plate_thick, -plate_thick])
            cube([plate_thick, ch_inside+2*ch_wall+2*plate_thick,
                  ch_deep+ch_wall+plate_thick+20]);

         // Cube for right angle tabs wrapped around rail
         translate([20, -(plate_thick+t_nut_spacing),
                    (ch_deep+ch_wall)-plate_thick])
            cube([plate_thick, ch_inside+2*ch_wall+2*plate_thick+2*t_nut_spacing,
                  plate_thick+20]);
      }

      // Subtract off slightly enlarged channel profile
      translate([plate_thick,0,0])
         channel(ch_inside-0.1, ch_wall+0.2, ch_deep+0.1, 150);

      // Subtract off holes for M5 screws to secure with t-nuts.

      translate([10,-(plate_thick+t_nut_spacing/2), 0])
         cylinder(r=5.2/2, h=50, $fn=64);

      translate([10,-(plate_thick+t_nut_spacing/2), ch_deep+ch_wall+10])
         rotate([0,90,0])
            cylinder(r=5.2/2, h=50, $fn=64);

      translate([10, ch_inside+2*ch_wall+plate_thick+t_nut_spacing/2, 0])
         cylinder(r=5.2/2, h=50, $fn=64);

      translate([10, ch_inside+2*ch_wall+plate_thick+t_nut_spacing/2,
                 ch_deep+ch_wall+10])
         rotate([0,90,0])
            cylinder(r=5.2/2, h=50, $fn=64);
   }
}

// It is tricky to get the dimensions right, so this just prints a small
// piece with the channel hole in it to test against the actual channel.
//
module testbracket() {
   rotate([0,-90,0])
   difference() {
      translate([0,-plate_thick,-plate_thick])
         cube([plate_thick,ch_inside+2*ch_wall+2*plate_thick,
               ch_deep+ch_wall+2*plate_thick]);
      translate([-1,0,0])
         channel(ch_inside-0.1, ch_wall+0.2, ch_deep+0.1, 150);
   }
}


// Subtract this from the power supply cover to leave a hole for mounting the
// A/C cable plug.
//
module plugshape(thick=3) {
   plugx=19.6;
   plugy=28;

   longbase=17;
   shortbase=14.9;

   holedia=3.7;
   holesep=39.4;
   union() {
      intersection() {
         rounded_corner_plate(plugx,plugy,thick,4);

         linear_extrude(height=thick)
            polygon([[-plugx/2, -plugy/2],
                     [(-plugx/2)+shortbase, -plugy/2],
                     [plugx/2, -longbase/2],
                     [plugx/2, longbase/2],
                     [(-plugx/2)+shortbase, plugy/2],
                     [-plugx/2, plugy/2]]);
      }
      translate([0,-holesep/2,0])
         cylinder(r=holedia/2, h=thick, $fn=64);
      translate([0,holesep/2,0])
         cylinder(r=holedia/2, h=thick, $fn=64);
   }
}

// Cover for the end of the Meanwell NES-350-12 29A power supply I'm
// using. Holes provided for the plug and switch salvaged from an old
// computer power supply and for the 12V cable to come out.
//
module power_cover() {
   pwidth=115.15;
   pheight=49;
   pdeep=45;
   pwall=4;
   ndeep=3.2;
   nheight=26;
   cwall=2;

   sw_high=13.2;
   sw_wide=19.9;
   sw_inner=21.2;

   union() {
      // shift things to orient the hole correctly since I got
      // the layout mirrored below :-).
      translate([pwidth,0,0])
      mirror([1,0,0])
      difference() {
         // Cube for the face of the cover
         cube([pwidth,pheight,pwall]);

         // Subtract off hole for the A/C plug connector.
         translate([(-(39.4+3.7)/2)+pwidth-8,(-19.6/2)+pheight-8,-1])
            rotate([0,0,-90])
               plugshape(pwall+2);

         // Subtract off hole for power switch to clip into
         translate([35,(pheight-sw_high)-8,-1])
            union() {
               translate([(sw_inner-sw_wide)/2,0,0])
                  cube([sw_wide,sw_high,3]);
               translate([0,0,2])
                  cube([sw_inner,sw_high,pwall+1]);
            }

         // Subtract off hole for 12V cable to come through
         translate([20, pheight-(10/2)-8, -1])
            cylinder(r=10/2, h=pwall+2, $fn=32);

         // Subtract off hole to see power LED through
         translate([12.15, 17,-1])
            cylinder(r=4/2, h=pwall+2, $fn=32);
      }

      // Add in the top wall
      translate([0,pheight-pwall,0])
         union() {
            cube([pwidth,pwall,pdeep]);
            translate([cwall,0,0])
               cube([pwidth-2*cwall,pwall,pdeep+ndeep]);
         }

      // Add in bottom wall
      cube([pwidth,pwall,pdeep]);

      // Add in side walls
      cube([pwall, pheight, pdeep]);
      translate([cwall,pheight-nheight,0])
         cube([pwall-cwall, nheight, pdeep+ndeep]);

      translate([pwidth-pwall,0,0]) cube([pwall, pheight, pdeep]);
      translate([pwidth-2*cwall,pheight-nheight,0])
         cube([pwall-cwall, nheight, pdeep+ndeep]);
   }
}

// These brackets hook around end of power_cover and reach the M4 screw holes
// on the power supply to use to secure the cover to the power supply. They
// can be glued on to the cover for greater security.
//
module power_brackets() {
   hole_to_cover=1810*(25.4/600.0); // center of hole to edge of cover

   difference() {
      union() {
         cube([4+hole_to_cover+2+4, 4, 3*4]);
         cube([4,(4+(138*(25.4/600.0)))-2, 3*4]);
      }
      translate([4+hole_to_cover,-1,(3*4)/2])
         rotate([-90,0,0])
            cylinder(r=4.2/2, h=6, $fn=64);
   }
}

// *************************************************************** Visualization
//
// Visualization modules used to render and examine 3D view.

// A representation of the layout of the microswitch so I can include it
// in the carriage assembly picture and check how it fits.
//
module microswitch() {
   mirror([0,1,0])
      difference() {
         union() {
            color("black") cube([478*mm_per_dot,256*mm_per_dot,switch_thick]);
            color("red") translate([((459-56)+(110/2))*mm_per_dot,
                       ((23-232)+(110/2))*mm_per_dot,
                       0])
               cylinder(r=(110*mm_per_dot)/2, h=switch_thick, $fn=64);
            color("silver") translate([(94-56)*mm_per_dot,
                       (475-232)*mm_per_dot,
                       0])
               cube([19*mm_per_dot,95*mm_per_dot,switch_thick]);
            color("silver") translate([(300-56)*mm_per_dot,
                       (479-232)*mm_per_dot,
                       0])
               cube([19*mm_per_dot,99*mm_per_dot,switch_thick]);
            color("silver") translate([(475-56)*mm_per_dot,
                       (479-232)*mm_per_dot,
                       0])
               cube([19*mm_per_dot,100*mm_per_dot,switch_thick]);
         }
         translate([((151-56)+(58/2))*mm_per_dot,
                    ((389-232)+(58/2))*mm_per_dot,
                    -1])
            cylinder(r=(58*mm_per_dot)/2, h=switch_thick+2, $fn=64);
         translate([((375-56)+(62/2))*mm_per_dot,
                    ((389-232)+(62/2))*mm_per_dot,
                    -1])
            cylinder(r=(62*mm_per_dot)/2, h=switch_thick+2, $fn=64);
      }
}

module switch_assembly() {
   union() {
      switch_holder();

      translate([6,0,-22.3])
         rotate([0,-90,0])
            mirror([0,1,0])
               microswitch();
   }
}

module low_profile_m5(length) {
   union() {
      color("gray") cylinder(r=5/2,h=length+lp_thick,$fn=64);
      color("gray") cylinder(r=lp_dia/2,h=lp_thick,$fn=64);
   }
}

// A wheel spacer
//
module spacer() {
   difference() {
      color("silver") cylinder(r=10/2,h=wheel_spacer,$fn=64);
      translate([0,0,-1])
         cylinder(r=wheel_min_dia/2,h=wheel_spacer+2, $fn=64);
   }
}

// An assembled mini v-wheel with spacers and bolt
//
module wheel_assembly() {
   union() {
      color("olive") mini_v_wheel();
      translate([0,0,wheel_thick/2])
         spacer();
      translate([0,0,-(wheel_spacer+wheel_thick/2)])
         spacer();
      translate([0,0,-(wheel_thick/2+wheel_spacer+plate_thick+lp_thick)])
         low_profile_m5(35);
      translate([0,0,m5nut_thick+plate_thick+wheel_spacer+wheel_thick/2])
         difference() {
            color("gray") render() hexnut(m5nut_flat,m5nut_thick);
            translate([0,0,-1])
               cylinder(r=5/2,h=m5nut_thick+2,$fn=64);
         }
   }
}

// An assembled X carriage without X rail
//
module x_assembly() {
   union() {
      translate([0,0,-(wheel_spacer+wheel_thick/2)])
         rotate([0,180,0])
            x_carriage(do_top=false);
      translate([0,0,(wheel_spacer+wheel_thick/2)])
         x_carriage();
      translate([0,wheel_y_sep,0])
         rotate([0,180,0])
            wheel_assembly();
      translate([-wheel_x_sep,-wheel_y_sep,0])
         rotate([0,180,0])
            wheel_assembly();
      translate([wheel_x_sep,-wheel_y_sep,0])
         rotate([0,180,0])
            wheel_assembly();
      translate([0,-(plate_y/2+e3d_offset+8),
                  -(wheel_thick/2 + wheel_spacer + plate_thick+3+10)])
         rotate([0,0,-90])
            union() {
               e3d_mount();
               translate([0,0,
                  (e3d_top_thick+e3d_mid_thick+e3d_bot_thick)-(62.3-e3d_up)])
               union() {
                  cylinder(r1=0.5,r2=4,h=4,$fn=64);
                  translate([0,0,4])
                     cylinder(r=4,h=62.3-4,$fn=64);
               }
            }
   }
}

// An assembled Y carriage with the Y or X rail
//
module y_assembly(do_right=true) {

   smooth_y=(belt_on_gear/2)      // out from center of drive gear
            + (smooth_idler/2);   // in to center of smooth idler pulley
   smooth_x=(belt_ysep/2)         // position belt to join X carriage
            - (smooth_idler/2);

   pulley_y=(belt_on_gear/2)      // out from center of drive gear
            -(belt_on_idler/2);   // back in to center of geared idler
   pulley_x=(belt_ysep/2)         // position belt to join X carriage
            - (belt_on_idler/2);

   low_belt=wheel_spacer          // height of bottom of lowest belt 
            + wheel_thick/2
            + plate_thick
            + above_low;
   high_belt=low_belt+(above_high-above_low);

   mirror([0,do_right?0:1,0])
   union() {
      // Flip top carriage over to really be on top
      translate([0,0,wheel_spacer+wheel_thick/2])
         rotate([0,180,0])
            translate([0,0,-plate_thick])
               y_carriage(do_top=true);
      // Stick bottom carriage on the bottom
      translate([0,0,-(plate_thick+wheel_spacer+wheel_thick/2)])
         y_carriage(do_top=false);
      // Put the middle wheel out at the edge
      translate([0,wheel_y_sep,0])
         rotate([0,180,0])
            wheel_assembly();
      // Put the other wheels at either end of inner edge
      translate([-wheel_x_sep,-wheel_y_sep,0])
         rotate([0,180,0])
            wheel_assembly();
      translate([wheel_x_sep,-wheel_y_sep,0])
         rotate([0,180,0])
            wheel_assembly();
      // Run bolt through smooth pulley hole
      translate([-smooth_x, -smooth_y, wheel_spacer+wheel_thick/2])
         low_profile_m5(30);
      // Run bolt through geared pulley hole
      translate([pulley_x, pulley_y, wheel_spacer+wheel_thick/2])
         low_profile_m5(30);

      // Represent the smooth pulley itself
      translate([-smooth_x, -smooth_y, do_right?high_belt:low_belt])
         rotate([0,0,90])
         pulley_and_belt(belt_on_idler-2*belt_thick, 5, belt_width, belt_thick,
                         do_right?"blue":"red", 90);

      // Show the geared pulley itself
      translate([pulley_x,pulley_y,do_right?low_belt:high_belt])
         pulley_and_belt(belt_on_gear-2*belt_thick, 5, belt_width, belt_thick,
                         do_right?"red":"blue", 90);

      // Add in the carriage cap
      %translate([0,0,37.5])
         rotate([0,-180,0])
            carriage_cap();

      // See if we can get the limit switch assembly to show up in the
      // right place.

      translate([-24,-42,-11])
         switch_assembly();
   }
}

module stacked_pulley_assembly() {
   belt_y=(vslot_size/2)+(belt_on_gear/2);
   hole_y=belt_y-(belt_on_idler/2);
   hole_x=plate_thick+(geared_idler_dia/2);

   union() {
      stacked_pulley_bracket();
      translate([hole_x, hole_y, plate_thick+above_low])
         rotate([0,0,90])
            pulley_and_belt(belt_on_gear-2*belt_thick, 5, belt_width, belt_thick,
                            "red", 90);
      translate([hole_x, hole_y, plate_thick+above_high])
         rotate([0,0,90])
            pulley_and_belt(belt_on_gear-2*belt_thick, 5, belt_width, belt_thick,
                            "blue", 90);
   }
}

module xy_motor_assembly(offset, color) {
   union() {
      xy_motor_mount();
      translate([0,0,offset])
         rotate([0,0,-90])
         pulley_and_belt(belt_on_gear-2*belt_thick, 5, belt_width, belt_thick,
                         color, 180);
   }
}

// Try to assemble the entire thing. Parameters determine how long the
// different rails are so I can render the model differently and see
// how much room I have. (And it definitely seems to take a lot of space
// to provide a 10" cube build area :-).
//
module shebang(x_rail_len=25.4*16,
               x_motion=0,
               y_rail_len=25.4*16,
               y_motion=0,
               z_above=25.4*3,
               z_below=25.4*16.5) {

   y_offset = x_rail_len/2+20/2+2;
   bed_offset=(plate_y/2 + e3d_offset+8);
   union() {
      translate([0,y_motion,0])
         union() {
            translate([x_motion,0,0])
               x_assembly();
            color("silver") translate([-x_rail_len/2,0,0])
               rotate([0,90,0])
                  vslot20x20(x_rail_len);
            translate([y_offset,0,0])
               rotate([0,0,-90])
                  y_assembly(do_right=true);
            translate([-y_offset,0,0])
               rotate([0,0,-90])
                  y_assembly(do_right=false);
         }
      color("silver") translate([y_offset,y_rail_len/2,0])
         rotate([90,0,0])
            vslot20x20(y_rail_len);
      color("silver") translate([-y_offset,y_rail_len/2,0])
         rotate([90,0,0])
            vslot20x20(y_rail_len);
      color("silver") translate([y_offset,20/2+y_rail_len/2,-z_below])
         vslot20x20(z_above+z_below);
      color("silver") translate([-y_offset,20/2+y_rail_len/2,-z_below])
         vslot20x20(z_above+z_below);
      color("silver") translate([y_offset,-(20/2+y_rail_len/2),-z_below])
         vslot20x20(z_above+z_below);
      color("silver") translate([-y_offset,-(20/2+y_rail_len/2),-z_below])
         vslot20x20(z_above+z_below);
      translate([y_offset,(-y_rail_len/2)+nema_motor_width/2,10+20])
         rotate([0,0,-90])
            xy_motor_assembly(-8, "blue");
      translate([-y_offset,(-y_rail_len/2)+nema_motor_width/2,10+20])
         rotate([0,0,-90])
            xy_motor_assembly(-15, "red");
      color("red")
         translate([-(25.4*10)/2, -(25.4*10)/2 - bed_offset, -(80-e3d_up)])
         cube([25.4*10,25.4*10,2]);
      translate([y_offset-10,y_rail_len/2,wheel_thick/2 + wheel_spacer])
         rotate([0,0,-90])
            stacked_pulley_assembly();
      mirror([1,0,0])
         translate([y_offset-10,y_rail_len/2,wheel_thick/2 + wheel_spacer])
            rotate([0,0,-90])
               stacked_pulley_assembly();
      // Show representation of the acme screws for the Z axis where they
      // would be if the motor was positioned just inside the bottom rail
      // (which assumes there will be a bottom rail :-)
      translate([y_offset-(nema_motor_width/2),0,(-z_below)+(89.2-17.76)])
         cylinder(r=4,h=290,$fn=64);
      translate([-(y_offset-(nema_motor_width/2)),0,(-z_below)+(89.2-17.76)])
         cylinder(r=4,h=290,$fn=64);
   }
}

// Redesigned z carriage to provide three wheels at each corner of the
// printbed. Old design was hopeless, this makes each corner of the
// printbed as strong as the x and y carriages (and with the lead screws
// also supporting each side, should be rock solid).
//
// The inner plates have attachments for the v-slot rails of the
// print bed, the outer plates just have holes for the wheels.
//
// Mout these with the single wheel on the inside of the printer
// to provide max z axis travel.
//
module z_carriage(do_inner=true,do_zstop=false) {
   extra_space = 0.2;
   inner_cube = 20+extra_space;
   outer_cube = inner_cube + 2*plate_thick;
   difference() {
      union() {
         // The basic plate for the carriage
         rounded_corner_plate(plate_x,plate_y,plate_thick,outer_wall/2);
         if (do_inner) {
            // The support where the printbed rail attaches.
            difference() {
               translate([-outer_cube/2,-outer_cube/2,0])
                  cube([outer_cube,outer_cube,plate_thick+20]);
               // Subtract off wall next to eccentric spacer to leave room
               // for spacer (only grab rail on 3 sides, not 4).
               translate([-(outer_cube+2)/2,(inner_cube/2)-0.01,-1])
                  cube([outer_cube+2,plate_thick+2,2+plate_thick+20]);
            }
         }
         if (do_zstop) {
            // switch_thick - thickness of the microswitch, and I
            // want the screw hole for the z-stop adjustment to be
            // exactly half that distance inside from the edge of the v-slot
            // rail.
            //
            // m3nut_flat is the width of the m3 nut, so I want the
            // block holding it to have about 2.5 mm of support on each
            // side of it.
            //
            // measuring the position the switch winds up at on the
            // extended z-stop piece, I see I want the screw hole to be
            // 20 mm from the center of the v-slot rail (which will need
            // to involve computation of the position of the rail under
            // the z carriage plate).
            //
            stop_hole_y=switch_thick/2-10;
            stop_width=m3nut_flat+2*2.5;
            stop_hole_height=20 - ((wheel_thick/2)+wheel_spacer);
            stop_nut_x=(plate_x/2)-((m3nut_thick+0.4)/2)-5;
            difference() {
               translate([-plate_x/2,stop_hole_y-(stop_width/2),0])
                  cube([10,stop_width,plate_thick+stop_hole_height]);

               translate([0,stop_hole_y,stop_hole_height])
                  rotate([0,-90,0])
                     cylinder(r=3.2/2,h=50,$fn=64);

               translate([-stop_nut_x,stop_hole_y,stop_hole_height])
               rotate([0,90,0])
               union() {
                  hexnut(m3nut_flat+0.4,m3nut_thick+0.4);
                  translate([-50,-(m3nut_flat+0.4)/2,-(m3nut_thick+0.4)])
                     cube([50,m3nut_flat+0.4,m3nut_thick+0.4]);
               }
            }
         }
      }
      // Subtract off hole for center wheel (uses eccentric spacer)
      // NOTE: The spacer couldn't quite be adjusted enough to make full
      // contact with some of the v-slots I tried with my test print, so
      // move it slightly closer to the center of the plate. (It was weird,
      // rotating the v-slot rail 90 degrees worked, rotate 90 again,
      // and it is too loose).
      translate([0,wheel_y_sep-0.3,-1])
         cylinder(r=wheel_max_dia/2,h=plate_thick+2,$fn=64);
      // Subtract off holes for other wheels (normal spacers)
      translate([wheel_x_sep,-wheel_y_sep,-1])
         cylinder(r=wheel_min_dia/2,h=plate_thick+2,$fn=64);
      translate([-wheel_x_sep,-wheel_y_sep,-1])
         cylinder(r=wheel_min_dia/2,h=plate_thick+2,$fn=64);
      // subtract off space for lock nut from z-stop nut trap (no effect
      // if no z-stop, but doesn't hurt anything).
      translate([-wheel_x_sep,-wheel_y_sep,plate_thick])
         cylinder(r=(9.5/2),h=20,$fn=64);
      if (do_inner) {
         // Subtract off opening for printbed rail to fit through
         translate([-inner_cube/2,-inner_cube/2,-1])
            cube([inner_cube,inner_cube,40]);
         // Subtract off the holes for the screws to attach to the
         // printbed rail.
         translate([plate_x/2,0,plate_thick+10])
            rotate([0,-90,0])
               cylinder(r=wheel_min_dia/2,h=plate_x,$fn=64);
         rotate([0,0,90])
            translate([plate_x/2,0,plate_thick+10])
               rotate([0,-90,0])
                  cylinder(r=wheel_min_dia/2,h=plate_x,$fn=64);
      }
   }
}

// Given a square region, fill it with cylinders (to be subtracted from
// the region to leave holes). Used in the y_chain_mount plate.
//
module swiss (x,y,xlen,ylen) {
   hole_dia=swiss_hole_dia;
   hole_sep=swiss_hole_sep;
   num_x = floor((xlen - 5)/hole_sep);
   num_y = floor((ylen - 5)/hole_sep);
   xoff = (xlen - (num_x * hole_sep))/2;
   yoff = (ylen - (num_y * hole_sep))/2;
   for ( i = [x+xoff : hole_sep : x+xoff+(hole_sep * num_x)]) {
      for ( j = [y+yoff : hole_sep : y+yoff+(hole_sep * num_y)]) {
         translate([i,j,-1])
            cylinder(r=(hole_dia/2),h=10,$fn=64);
      }
   }
}

// Design a plate with lots of mounting holes to go on the outside
// of the right y carriage to provide a place to mount a cable chain.
//
// Printed this and it seems to work fine, I can push fit it on the
// existing printer carriage for testing.
//
// This just gives me places to mount stuff on the side, still need to
// design the actual coupling to bolt to this plate to connect the cable
// chain.

module y_chain_plate() {
   w=y_chain_plate_width;
   flange=3;
   grab=y_chain_plate_grab;
   top_extra=30;
   bottom_extra=30;
   cxm=cap_extra_mult;
   carriage_thick=(2*wheel_spacer)+wheel_thick+(2*plate_thick);
   support_height=3*pulley_spacer+2*geared_idler_thick+plate_thick;
   cap_wide=carriage_cap_width;
   cap_extra=(plate_y-cap_wide)/2;
   difference() {
      union() {
         // main body - long cube that will be vertically attached to the
         // outside edge of the right y carriage
         cube([w,top_extra+bottom_extra+carriage_thick+support_height,
               plate_thick]);
         // thicker part for the top because the carriage cap is slightly
         // indented from the main carriage pieces
         translate([0,bottom_extra+carriage_thick,0])
            cube([w,top_extra+support_height,plate_thick+cap_extra]);
         // Little "tabs" that stuck out to grab hold of the top and bottom
         // carriage pieces as well as the carriage cap.
         translate([0,bottom_extra-flange,0])
            cube([w,flange,grab+plate_thick]);
         translate([0,plate_thick+bottom_extra,0])
            cube([w,flange,grab+plate_thick]);
         translate([0,bottom_extra+carriage_thick-(flange+plate_thick),0])
            cube([w,flange,grab+plate_thick]);
         translate([0,plate_thick+bottom_extra+carriage_thick-plate_thick,0])
            cube([w,flange,grab+plate_thick]);
         translate([0,bottom_extra+carriage_thick+support_height,0])
            cube([w,flange,grab+plate_thick+cap_extra]);
         translate([0,bottom_extra+carriage_thick+support_height-
                         (plate_thick+flange),0])
            cube([w,flange,grab+plate_thick+cap_extra]);
      }
      // punch holes through all the flat areas on top, bottom, and between
      // carriage plates. I know I want to mount the cable chain with these,
      // and perhaps other things will need mounting someday, so might as
      // well add as many holes as will fit.
      swiss(0,0,w,bottom_extra-flange);
      swiss(0,bottom_extra+plate_thick+flange,w,
            carriage_thick-(2*(plate_thick+flange)));
      swiss(0,bottom_extra+carriage_thick+flange,w,
            (support_height - plate_thick)-(2*flange));
      swiss(0,bottom_extra+carriage_thick+support_height+flange,w,
            top_extra-flange);
      // Cut cylinder out of the "flanges" to leave space for
      // one of the carriage cap supports it would otherwise collide with.
      translate([10,bottom_extra+carriage_thick-flange,
                 plate_thick+cap_extra+(9/2)])
         rotate([-90,0,0])
            union() {
               translate([-(9.2/2),-20,0])
                  cube([9.2,20,support_height+2]);
               cylinder(r=(9.2/2),h=support_height+2,$fn=64);
            }
      // Cut screw holes through all the "flanges" for screwing the
      // plate to the carriage (don't know if I'll add screws everywhere,
      // but I'll have the holes if I want to). These will need corresponding
      // holes in the y carriage pieces.
      translate([3,0,plate_thick+(cap_extra*cxm)+(grab/2)])
         rotate([-90,0,0])
            cylinder(r=(swiss_hole_dia/2),h=100,$fn=64);
      translate([w-3,0,plate_thick+(cap_extra*cxm)+(grab/2)])
         rotate([-90,0,0])
            cylinder(r=(swiss_hole_dia/2),h=100,$fn=64);
   }
}

// Rotate and position the same holes used for the bottom of the y carriage
// cable chain mounting plate so they line up to be used to bolt the
// cable chain clip in place.
//
module y_clip_holes() {
   translate([-1,-24,17])
   rotate([0,90,0])
   swiss(0,0,25,27);
}

// Take the end2 clip extracted from over in the cable chain directory
// and tack on supports to allow this to be bolted to the y carriage
// mounting plate at the correct 40 degree angle.
//
module y_carriage_chain_clip() {
   rotate([0,180,0])
   rotate([40,0,0])
   difference() {
      union() {
         rotate([-40,0,0])
         translate([0,21.5,0])
         rotate([0,180,0])
         rotate([0,0,180])
         import("end2-clip.stl");
         difference() {
            rotate([0,90,0])
               cylinder(r=25,h=2.5,$fn=64);
            translate([-1,-50,-32])
               cube([10,100,25]);
            rotate([-40,0,0])
               translate([-1,-100,0])
                  cube([10,200,50]);
         }
         translate([0,-15,-4.7])
            rotate([-40,0,0])
               cube([2.5,15,10]);
      }
      y_clip_holes();
   }
}

// Holes (actually cylinders) the same size and separation as the holes
// in the cable chain end pieces.
//
module chain_end_holes() {
   rad=3.4/2;
   union() {
      cylinder(r=rad,h=25,$fn=64);
      translate([0,10,0])
         cylinder(r=rad,h=25,$fn=64);
   }
}

// Piece that can screw onto the x carriage and provide a place to
// screw a cable chain end onto that.
//
module x_chain_mount() {
   extra=50;
   xh=wheel_thick+(2*wheel_spacer)+2*(plate_thick+3+10);
   wide=13.3;
   fy=3;
   ly=3+5;
   fz=(xh/2)-5;
   lz=fz-5;
   rotate([0,-90,0])
      difference() {
         union() {
            translate([0,0,-xh/2])
               cube([plate_thick,wide,xh+extra]);
            translate([0,-(plate_thick-0.01),xh/2])
               cube([22.5+plate_thick,plate_thick,extra]);
         }
         translate([plate_thick*2,-(plate_thick+1),(xh/2)-0.1])
            rotate([0,30,0])
               cube([30,plate_thick+2,100]);
         for (i = [fy : 5 : ly]) {
            for (j = [fz : -5: lz]) {
               translate([-1,i,j])
                  rotate([0,90,0])
                     cylinder(r=(3.2/2),h=plate_thick+2,$fn=64);
               translate([-1,i,-j])
                  rotate([0,90,0])
                     cylinder(r=(3.2/2),h=plate_thick+2,$fn=64);
            }
         }
         translate([plate_thick+(22.5/2),5,(xh/2)+extra-15])
            rotate([90,0,0])
               chain_end_holes();
         translate([-5,7.4,(xh/2)+extra-15])
            rotate([90,0,0])
               rotate([0,90,0])
                  chain_end_holes();
      }
}

// Bolts to y carriage plate, and holds a cable chain end to connect the
// chain from the x carriage. (Has a tendency to curl up, hence the
// strain relief holes in the bottom edge).
//
module yx_chain_mount() {
   rad=6;
   wid=(y_chain_plate_width-5.05)+22.5+plate_thick;
   difference() {
      translate([0,0,wid])
      rotate([0,180,0])
      difference() {
         union() {
            cylinder(r=rad, h=wid, $fn=64);
            translate([rad-plate_thick,-80,0])
               cube([plate_thick,80,wid]);
            translate([-40,rad-plate_thick,0])
               cube([40,plate_thick,wid]);
            translate([-40,0,0])
               cylinder(r=rad, h=wid, $fn=64);
            translate([rad-plate_thick,-(25+(rad-plate_thick)),wid-plate_thick])
            difference() {
               cube([25+plate_thick,25,plate_thick]);
                  translate([13,10,-10])
                     rotate([0,0,-45])
                        chain_end_holes();
            }
         }
         translate([0,0,-1])
            cylinder(r=(rad-plate_thick),h=wid+2,$fn=64);
         translate([-40,0,-1])
            cylinder(r=(rad-plate_thick),h=wid+2,$fn=64);
         translate([-40,-((2*rad)-(rad-plate_thick)),-1])
            cube([40,2*rad,wid+2]);
         translate([(rad-plate_thick)-20,-20,-1])
            cube([20,20,wid+2]);
         translate([-50,-20,-1])
            cube([20,20,wid+2]);
         translate([rad,0,0])
            rotate([0,-90,0])
               translate([rad-plate_thick,-80,0])
                  swiss(0,0,y_chain_plate_width,27);
         translate([-50,-50,(wid-(22.5+plate_thick))-100])
            cube([100,100,100]);
      }
      // cut some holes along the long bottom pieces for
      // strain relief so it is less likely to curl up
      for (i = [-33 : -10 : -73 ]) {
         translate([0,i,0])
            rotate([0,-90,0])
               cylinder(r=1.5,h=10,$fn=64);
      }
      for (j = [6 : 10 : 40 ]) {
         translate([j,0,0])
            rotate([-90,0,0])
               cylinder(r=1.5,h=10,$fn=64);
      }
      // And punch some holes in the big blank plates for
      // more strain relief (and to use less plastic).
      translate([0,-10,13])
         rotate([0,-90,0])
            cylinder(r=6,h=10,$fn=64);
      translate([0,-40,13])
         rotate([0,-90,0])
            cylinder(r=6,h=10,$fn=64);
      translate([20,0,13])
         rotate([-90,0,0])
            cylinder(r=8,h=10,$fn=64);
   }
}

// Show enough carriage assembly to see if things are in the right place...
//
module carriage_assembly() {

   smooth_y=(belt_on_gear/2)      // out from center of drive gear
            + (smooth_idler/2);   // in to center of smooth idler pulley
   smooth_x=(belt_ysep/2)         // position belt to join X carriage
            - (smooth_idler/2);

   pulley_y=(belt_on_gear/2)      // out from center of drive gear
            -(belt_on_idler/2);   // back in to center of geared idler
   pulley_x=(belt_ysep/2)         // position belt to join X carriage
            - (belt_on_idler/2);

   low_belt=wheel_spacer          // height of bottom of lowest belt 
            + wheel_thick/2
            + plate_thick
            + above_low;
   high_belt=low_belt+(above_high-above_low);

   union() {
      // Flip top carriage over to really be on top
      translate([0,0,wheel_spacer+wheel_thick/2])
         rotate([0,180,0])
            translate([0,0,-plate_thick])
               y_carriage(do_top=true);
      // Stick bottom carriage on the bottom
      translate([0,0,-(plate_thick+wheel_spacer+wheel_thick/2)])
         y_carriage(do_top=false);
      // Put the middle wheel out at the edge
      translate([0,wheel_y_sep,0])
         rotate([0,180,0])
            wheel_assembly();
      // Put the other wheels at either end of inner edge
      translate([-wheel_x_sep,-wheel_y_sep,0])
         rotate([0,180,0])
            wheel_assembly();
      translate([wheel_x_sep,-wheel_y_sep,0])
         rotate([0,180,0])
            wheel_assembly();
      // Run Y rail down center
      rotate([0,90,0])
         translate([0,0,-50])
            color("silver") vslot20x20(100);
      // Run X rail out side
      translate([0,-12,0])
         rotate([90,0,0])
            color("silver") vslot20x20(150);
      // Run bolt through smooth pulley hole
      translate([-smooth_x, -smooth_y, wheel_spacer+wheel_thick/2])
         low_profile_m5(30);
      // Run bolt through geared pulley hole
      translate([pulley_x, pulley_y, wheel_spacer+wheel_thick/2])
         low_profile_m5(30);

      // Show high belt coming from motor to smooth pulley
      translate([-smooth_x,-belt_on_gear/2,high_belt])
         color("blue") cube([100,belt_thick,belt_width]);
      // Wrap it 90 degrees around smooth pulley
      translate([-smooth_x, -smooth_y, high_belt])
         difference() {
            color("blue")
               cylinder(r=belt_thick+smooth_idler/2,h=belt_width,$fn=64);
            translate([0,0,-1])
               cylinder(r=smooth_idler/2,h=belt_width+2,$fn=64);
            translate([0,-smooth_idler,-1])
               cube([2*smooth_idler,2*smooth_idler,belt_width+2]);
            translate([-smooth_idler,-2*smooth_idler,-1])
               cube([2*smooth_idler,2*smooth_idler,belt_width+2]);
         }
      // And send it off towards the X carriage
      translate([-(smooth_x+belt_thick+smooth_idler/2),-(80+smooth_y),high_belt])
         color("blue") cube([belt_thick,80,belt_width]);
      // High belt on other side of drive gear headed for other end of printer
      translate([-100,(belt_on_gear/2)-belt_thick,high_belt])
         color("blue") cube([200,belt_thick,belt_width]);
      // Represent the smooth pulley itself
      translate([-smooth_x, -smooth_y, high_belt])
         color("silver") cylinder(r=smooth_idler/2,h=belt_width,$fn=64);

      // Low belt coming from back of printer to geared idler
      translate([pulley_x-100,(belt_on_gear/2)-belt_thick,low_belt])
         color("red") cube([100,belt_thick,belt_width]);
      // Wrap it around the pulley
      translate([pulley_x,pulley_y,low_belt])
      difference() {
         color("red") cylinder(r=belt_on_idler/2,h=belt_width,$fn=64);
         translate([0,0,-0.1])
            cylinder(r=(belt_on_idler/2)-belt_thick,h=belt_width+2,$fn=64);
         translate([-2*belt_on_idler,-belt_on_idler,-0.1])
            cube([2*belt_on_idler,2*belt_on_idler,belt_width+2]);
         translate([-belt_on_idler,-2*belt_on_idler,-0.1])
            cube([2*belt_on_idler,2*belt_on_idler,belt_width+2]);
      }
      // And send it off towards the X carriage
      translate([(pulley_x+belt_on_idler/2)-belt_thick,pulley_y-100,low_belt])
         color("red") cube([belt_thick,100,belt_width]);
      // Show the pulley itself
      translate([pulley_x,pulley_y,low_belt])
         color("silver")
            cylinder(r=(belt_on_idler/2)-belt_thick,h=belt_width,$fn=64);

      // Add in the carriage cap
      %translate([0,0,38.51])
         rotate([0,-180,0])
            carriage_cap();

      // Now drag the X carriage into position to see if the belts line
      // up correctly...

      translate([0,-100,0])
         rotate([0,0,90])
            x_assembly();

      // See if we can get the limit switch assembly to show up in the
      // right place.

      translate([-24,-42,-11])
         switch_assembly();

      // Show the cable chain mounting plate on the carriage

      #translate([10,plate_thick+plate_y/2,
                -(30+plate_thick+wheel_spacer+wheel_thick/2)])
         rotate([90,0,0])
            y_chain_plate();
   }
}
