include <carriages.scad>
include <hexnut.scad>

// Sigh: Without a fan, I can't print these parts well enough on Jiggit,
// so I'm printing the (hopefully) final versions on the solidoodle.

// Various dimensions used in these parts:

fanside=40.2;   // Square Noctua fan has sides this long (with .2 slop added)
fanthick=12.2;  // Likewise Noctua fan is this thick with .2 slop added
mounthole=4.2;  // sizeof mounting holes in fan corners
fanholesep=32;  // 4mm mounting holes in fan are separated by 32mm
bladedia=38.2;  // Hole fan blades spin in is this diameter
m2hole=2.2;     // I use M2 bolts, make holes this big for them
fanboxwall=3;   // Thickness of left and right wall of the fan trap box
beamthick=5;    // Thickness of the beams that run under the carriage
m2headthick=2.1; // how deep a hole needed for M2 bolt head
m2headdia=4;    // diameter hole to fit M2 bolt head
// Just a basic model of the fan I can use in visualizations
//
module noctua () {
   side=fanside;    // fan is this square (with .2 for slop)
   thick=fanthick;   // fan is this thick (with slop again)
   fandia=bladedia;  // diameter of hole fan lives in
   holedia=mounthole;  // mounting holes
   holesep=fanholesep;   // mounting hole separation
   difference() {
      cube([side,side,thick]);
      translate([side/2+holesep/2,side/2+holesep/2,-1])
         cylinder(r=holedia/2,h=thick+2,$fn=64);
      translate([side/2+holesep/2,side/2-holesep/2,-1])
         cylinder(r=holedia/2,h=thick+2,$fn=64);
      translate([side/2-holesep/2,side/2-holesep/2,-1])
         cylinder(r=holedia/2,h=thick+2,$fn=64);
      translate([side/2-holesep/2,side/2+holesep/2,-1])
         cylinder(r=holedia/2,h=thick+2,$fn=64);
      translate([side/2,side/2,-1])
         cylinder(r=fandia/2,h=thick+2,$fn=64);
   }
}

// This visualization shows that the fan will fit vertically under the
// carriage and still sit above the nozzle, so no tilting at angle is
// required.
//
//import("complete-carriage.stl");
//translate([-33,-(3+fanside/2),6+(-74+fanside)])
//rotate([0,90,0])
//noctua();

// The bolt for the wheel sticks down in the middle of the back of the
// carriage, perhaps the fan should be positioned in front of that.
// That would also make the carriage into an shield to prevent junk from
// falling into the fan.

// Shield to keep fingers out of the fan
//
module finger_shield() {
   sep=6;
   r1=(bladedia/2)-sep;
   rotate([0,0,45])
   union() {
      for (r = [r1:-sep:sep]) {
         difference() {
            cylinder(r=r,h=1,$fn=64);
            translate([0,0,-1])
               cylinder(r=r-1,h=3,$fn=64);
         }
      }
      cylinder(r=2,h=1,$fn=64);
      translate([-0.5,-(fanside-1)/2,0])
         cube([1,fanside-1,1]);
      rotate([0,0,90])
         translate([-0.5,-(fanside-1)/2,0])
            cube([1,fanside-1,1]);
   }
}

// Shape of slot for fan cable to come out of
//
module cable_slot(len=9) {
   cablepos=9.47; // distance from top of fan to cable position
   wall=fanboxwall;
   thick=fanthick+wall;   // fan is this thick
   move=-25;

   union() {
      translate([move,3+cablepos,wall+thick/2])
         cube([30,3,len]);
      translate([move,3+cablepos+(3/2),wall+thick/2])
         rotate([0,90,0])
            cylinder(r=3/2,h=30,$fn=64);
      translate([move,3+cablepos+(3/2),len+wall+thick/2])
         rotate([0,90,0])
            cylinder(r=3/2,h=30,$fn=64);
   }
}

// Box to enclose fan trapping it against shroud. This box is also the place
// to add whatever structure will be used to secure the rest of the fan
// assembly (like the shroud that poke out around the nozzle).
//
// If do_reg is true make the side registration bits bigger and make the
// side M2 holes into posts instead of holes (same for the fan cable slot).
// Set do_reg to true when subtracting this shape from the support beams.
//
module fan_trap(do_reg=false) {
   extra=do_reg?0.3:0;
   wall=fanboxwall;
   side=fanside+0.2;    // fan is this square (with extra .2 slop here)
   thick=fanthick+wall;   // fan is this thick
   fandia=bladedia;  // diameter of hole fan lives in
   bigdia=mounthole-0.2;   // M4 mounting hole size
   holesep=fanholesep;   // M4 mounting hole separation
   tinydia=m2hole;  // M2 mounting hole to use instead
   m2shift=sin(45)*(bigdia-tinydia);
   m2sep=fanholesep+m2shift;

   difference() {
      union() {
         difference() {
            // Cube bigger than fan 
            cube([side+2*wall,side+2*wall,wall+thick]);
            // Subtract fan shaped hole from it
            translate([wall,wall,wall])
               cube([side,side,thick+1]);
            // Subtract slot from side for cable to run through
            if (! do_reg) {
               cable_slot();
            }
            // Subtract bolt holes
            translate([(side+2*wall)/2+m2sep/2,(side+2*wall)/2+m2sep/2,-1])
               cylinder(r=tinydia/2,h=thick+2,$fn=64);
            translate([(side+2*wall)/2+m2sep/2,(side+2*wall)/2-m2sep/2,-1])
               cylinder(r=tinydia/2,h=thick+2,$fn=64);
            translate([(side+2*wall)/2-m2sep/2,(side+2*wall)/2-m2sep/2,-1])
               cylinder(r=tinydia/2,h=thick+2,$fn=64);
            translate([(side+2*wall)/2-m2sep/2,(side+2*wall)/2+m2sep/2,-1])
               cylinder(r=tinydia/2,h=thick+2,$fn=64);
            // Subtract fan hole
            translate([(side+2*wall)/2,(side+2*wall)/2,-1])
               cylinder(r=fandia/2,h=thick+2+2,$fn=64);
            // Not a lot of extra space under the carriage, cut the top of the
            // box down to just 1mm thick
            translate([-1,-1,-1])
               cube([side+2*wall+2,wall,wall+thick+2]);
            // Same problem at bottom, don't want it sticking below nozzle
            translate([-1,(side+2*wall)-(wall-1),-1])
               cube([side+2*wall+2,wall,wall+thick+2]);
         }
         if (do_reg) {
            // When doing registration with beams, add cable slot so it
            // will be subtracted from the beam
            cable_slot();
         }
         translate([(side+2*wall)/2,(side+2*wall)/2,0])
            finger_shield();
         // registration shapes for support beam on outside of fan trap
         translate([-(2+extra),4.5-(extra/2),4.5-(extra/2)])
            difference() {
               cube([3+extra,5+extra,10+extra]);
               translate([2+extra,0,0])
                  rotate([0,-45,0])
                     translate([-(5+extra),-1,0])
                        cube([5+extra,10,10]);
            }
         translate([side+2*wall,0,0])
            mirror([1,0,0])
               translate([-(2+extra),4.5-(extra/2),4.5-(extra/2)])
                  difference() {
                     cube([3+extra,5+extra,10+extra]);
                     translate([2+extra,0,0])
                        rotate([0,-45,0])
                           translate([-(5+extra),-1,0])
                              cube([5+extra,10,10]);
                  }
         if (do_reg) {
            translate([-12,(4.5-(extra/2))+((5+extra)/2),
                       (4.5-(extra/2))+((10+extra)/2)])
               rotate([0,90,0])
                  cylinder(r=2.2/2,h=70,$fn=64);
         }
      }
      if (! do_reg) {
         // put M2 hole all the way through
         translate([-12,4.5+(extra/2)+((5+extra)/2),
                    4.5+(extra/2)+((10+extra)/2)])
            rotate([0,90,0])
               cylinder(r=2.2/2,h=70,$fn=64);
         // put recess for M2 bolt head on inside
         translate([wall-m2headthick,4.5+(extra/2)+((5+extra)/2),
                    4.5+(extra/2)+((10+extra)/2)])
            rotate([0,90,0])
               cylinder(r=m2headdia/2,h=side+2*m2headthick,$fn=64);
      }
   }
}

// Visualize X carriage (and also use to subtract colliding bits from the
// support beams).
module visual_x_carriage() {
   union() {
      x_carriage(false);

      // Add in cylinders showing approx size of lock nuts we need to avoid
      translate([0,wheel_y_sep,0])
         cylinder(r=2+wheel_max_dia/2,h=plate_thick+5,$fn=64);
      translate([wheel_x_sep,-wheel_y_sep,0])
         cylinder(r=2+wheel_min_dia/2,h=plate_thick+5,$fn=64);
      translate([-wheel_x_sep,-wheel_y_sep,0])
         cylinder(r=2+wheel_min_dia/2,h=plate_thick+5,$fn=64);
   }
}

// Support that wraps over back of X carriage, attaches to fan trap and
// screws to front of X carriage (all while avoiding collisions with
// carriage parts).
module support_beam_back() {
   thick=beamthick;
   short=4;
   union() {
      translate([-thick/2,short-plate_y/2,plate_thick])
         cube([thick,plate_y-short,3+10]);
      difference() {
         translate([-thick/2,plate_y/2,0])
            rotate([0,90,0])
               cylinder(r=plate_thick+3+10,h=thick,$fn=64);
         translate([-(thick+2)/2,(plate_y/2)-(plate_thick+3+10),
                    -(plate_thick+3+10+1)])
            cube([thick+2,plate_thick+3+10,plate_thick+3+10+1]);
         translate([-(thick+2)/2,(plate_y/2)-1,-(plate_thick+3+10+1+3)])
            cube([thick+2,plate_thick+3+10+1,plate_thick+3+10+1]);
      }
      translate([-thick/2,plate_y/2,-3/2])
         rotate([0,90,0])
            cylinder(r=3/2,h=thick,$fn=64);
   }
}

module left_support_beam(do_fan_trap=true) {
   difference() {
      translate([-(3+beamthick/2+(fanside+0.2+2*fanboxwall)/2),0,0])
         union() {
            support_beam_back();
            translate([-beamthick/2,4-plate_y/2,plate_thick])
               cube([beamthick+8,4,10+3]);
            if (do_fan_trap) {
               translate([1.5,-9,plate_thick])
                  cube([7,20,10+3]);
            }
         }
      translate([-25,16-plate_y/2,plate_thick+3+5])
         rotate([90,0,0])
            cylinder(r=3.2/2,h=15,$fn=64);
      translate([-25,7-plate_y/2,plate_thick+3+5])
         rotate([90,0,0])
            hexnut(m3nut_flat+0.4,5);
      // Subtract the X carriage view from this to make holes for
      // the lock nuts on the wheels.
      visual_x_carriage();
      // Subtract the fan_trap to make holes for the M2 bolts and
      // registration guides.
      if (do_fan_trap) {
         translate([(-(fanside+0.2+2*fanboxwall)/2)+3,10,plate_thick])
            rotate([90,0,0])
               translate([0,-2,0])
                  fan_trap(do_reg=true);
      }
   }
}

module right_support_beam() {
   difference() {
      union() {
         mirror([1,0,0])
            translate([6,0,0])
               left_support_beam(false);
      }
      // Subtract the fan_trap to make holes for the M2 bolts and
      // registration guides.
      render() {
      translate([(-(fanside+0.2+2*fanboxwall)/2)-3,10,plate_thick])
         rotate([90,0,0])
            translate([0,-2,0])
               fan_trap(do_reg=true);
      }
   }
}


// General purpose ring creation for blowing air in a downward direction.
//
// dia = diameter of circle through center of ring
// wid = width of the tube shape that makes up the ring
// wall = how thick to make the ring walls
// extra = how much extra thickness to include where airholes go
// focus = how far below the ring the air holes should be pointed at
// squash = how much to squash the circular shape to make an oval
// blowhole = diameter of holes that blow air
// numholes = number of blowholes to make in ring
// solid - make version with no holes (solid for subtracting from other bits)
//
// Not based on any fluid dynamic principles - based on what I could
// figure out how to do in openscad :-).
//
module air_ring(dia=30,wid=8,wall=1.5,extra=1,focus=10,squash=0.7,
                blowhole=1.6, numholes=16, solid =false) {
   ang=atan(focus/(dia/2));
   hyp=sqrt((focus*focus)+((dia/2)*(dia/2)));
   holerot=360/numholes;

   
   difference() {
      // make outside of ring
      rotate_extrude($fn=64)
         translate([dia/2,0,0])
            rotate([0,0,ang])
               scale([1,squash,1])
                  union() {
                     if (extra > 0) {
                        translate([-((wid/2)+extra),-((wid-2*wall))/2])
                           square([(wid/2)+extra,wid-2*wall]);
                     }
                     circle(r=wid/2,$fn=64);
                  }
      // hollow out inside of ring
      if (! solid) {
         rotate_extrude($fn=64)
            translate([dia/2,0,0])
               rotate([0,0,ang])
                  scale([1,squash,1])
                     circle(r=(wid/2)-wall,$fn=64);
         // Subtract numholes from the inside of the ring for air to come out
         for (rot=[0:holerot:360]) {
            rotate([0,0,rot])
            translate([0,0,-focus])
            rotate([0,90-ang,0])
            union() {
               cylinder(r=(blowhole/2),h=hyp,$fn=64);
               translate([0,0,hyp-(extra+wid/2)])
               cylinder(r1=(blowhole/2),r2=squash*((wid/2)-wall),h=extra+wid/2,
                        $fn=64);
            }
         }
      }
   }
}

module sloped_cone(rad1,rad2,hi) {
   difference() {
      cylinder(r1=rad1,r2=rad2,h=hi,$fn=64);
      translate([0,12,hi])
         rotate([14,0,0])
            translate([-50/2,-50,0])
               cube([50,50,50]);
   }
}

// Fits above the fan in the fan_trap and holds the shroud that directs the
// air. Much work to do, but I know I want to start with a plate that fits
// over the fan and has holes to bolt it in, and I've visualized the
// size and position of the air_ring over in the complete-carriage.scad file,
// So now I have to work out how to join them together...
//
// This design may need to print with supports, but I'll probably try without
// them first to see if the angle is steep enough to print OK.
//
module shroud () {
   side=fanside;    // fan is this square (with .2 for slop)
   thick=3;      // shroud base is this thick
   bigdia=mounthole-0.2;   // M4 mounting hole size
   holesep=fanholesep;   // M4 mounting hole separation
   tinydia=m2hole;  // M2 mounting hole to use instead
   m2shift=sin(45)*(bigdia-tinydia);
   m2sep=fanholesep+m2shift;
   difference() {
      union() {
         difference() {
            // Cube that fits in fan_trap
            cube([side,side,thick]);
            // holes for bolts
            translate([side/2+m2sep/2,side/2+m2sep/2,-1])
               cylinder(r=tinydia/2,h=thick+2,$fn=64);
            translate([side/2+m2sep/2,side/2-m2sep/2,-1])
               cylinder(r=tinydia/2,h=thick+2,$fn=64);
            translate([side/2-m2sep/2,side/2-m2sep/2,-1])
               cylinder(r=tinydia/2,h=thick+2,$fn=64);
            translate([side/2-m2sep/2,side/2+m2sep/2,-1])
               cylinder(r=tinydia/2,h=thick+2,$fn=64);
         }
         // Position the ring that blows air so it is centered on
         // the nozzle and the holes for air point just below nozzle.
         difference() {
            translate([fanside/2,fanside-4.7,31.8])
               rotate([90,0,0])
                  air_ring(dia=49,focus=8,numholes=24,blowhole=1.8);
            // Subtract (slightly smaller) sloped code from ring to open
            // it up to the shroud.
            translate([side/2,side/2,0])
               sloped_cone((bladedia/2)-0.1,((0.95*bladedia)/2)-0.1,
                           (thick+8.6)-0.1);
         }
         // Now just need duct work to join the plate to the ring
         // ("just" he says).
         //
         // Let's try a short cone with sloped top (not too bad)
         difference() {
         translate([side/2,side/2,0])
            sloped_cone(bladedia/2,(0.95*bladedia)/2,thick+8.6);
         // Shape looks OK, need to subtract ring from cone and cone from ring
         // (with a bit of overlap) to hollow out the shroud (not to mention
         // smaller cone from inside of cone). I'll want a way to make a solid
         // ring to do the subtract - same for cone.

         // Subtract slightly squished solid ring from shroud to open up
         // shroud to the ring
         translate([fanside/2,fanside-4.7,31.8])
            rotate([90,0,0])
               scale([1,1,0.95])
                  air_ring(dia=49,focus=8,numholes=24,blowhole=1.8,solid=true);
         }
      }
      // Subtract smaller sloped cone from inside of larger sloped code.
      translate([side/2,side/2,0])
         translate([0,0,-0.01])
            sloped_cone((bladedia/2)-1.5,((0.95*bladedia)/2)-1.5,thick+8.6-1.5);
   }
}

// One of these "horns" is on each side of the hornshroud to blow cooling air.
//
module onehorn(rad=4.25,backlen=10,frontlen=30,wall=1.5,make_solid=false) {
   slotw=2;
   difference() {
      union() {
         rotate([0,90,0])
            cylinder(r=rad,h=backlen,$fn=64);
         translate([backlen,0,0])
            sphere(r=rad,$fn=64);
         translate([backlen,0,0])
            cylinder(r=rad,h=frontlen,$fn=64);
         translate([backlen,0,frontlen])
            sphere(r=rad,$fn=64);
      }
      if (! make_solid) {
         translate([-wall,0,0])
            onehorn(rad-wall,backlen+wall,frontlen,wall,true);
         // punch slot in horn for air (preview in complete-carriage
         // to watch with # prefixed cube points to adjust angle and position)
         translate([backlen,0,0])
            rotate([0,0,-33])
               translate([-30,-slotw/2,19])
                  cube([30,slotw,7]);
      }
   }
}

// The "head" the horns come out of
module onehead(side=fanside,blade=bladedia,down=30,rad=8.5,hi=12,rd=2,make_solid=false) {
   difference() {
      union() {
         intersection() {
            translate([-2,down,rad-rd])
               rotate([0,90,0])
                  cylinder(r=rad,h=50,$fn=64);
            translate([side/2,side/2,0])
               cylinder(r=blade/2,h=50,$fn=64);
         }
         translate([side/2,side/2,0])
            cylinder(r=blade/2,h=hi,$fn=64);
      }
      if (! make_solid) {
         onehead(side,blade-3,down,rad-1.5,hi-1.5,1,true);
      }
   }
}

// Don't like the ring shroud, lets try something completely different
//
module hornshroud() {
   side=fanside;    // fan is this square (with .2 for slop)
   thick=3;      // shroud base is this thick
   bigdia=mounthole-0.2;   // M4 mounting hole size
   holesep=fanholesep;   // M4 mounting hole separation
   tinydia=m2hole;  // M2 mounting hole to use instead
   m2shift=sin(45)*(bigdia-tinydia);
   m2sep=fanholesep+m2shift;
   difference() {
      // Cube that fits in fan_trap
      cube([side,side,thick]);
      // holes for bolts
      translate([side/2+m2sep/2,side/2+m2sep/2,-1])
         cylinder(r=tinydia/2,h=thick+2,$fn=64);
      translate([side/2+m2sep/2,side/2-m2sep/2,-1])
         cylinder(r=tinydia/2,h=thick+2,$fn=64);
      translate([side/2-m2sep/2,side/2-m2sep/2,-1])
         cylinder(r=tinydia/2,h=thick+2,$fn=64);
      translate([side/2-m2sep/2,side/2+m2sep/2,-1])
         cylinder(r=tinydia/2,h=thick+2,$fn=64);
      // hole in plate for fan to blow through
      translate([side/2,side/2,-1])
         cylinder(r=bladedia/2,h=thick+2,$fn=64);
   }
   // That much is always the same, now see if I can build a couple of
   // "horns" that poke around the hot end to be used to blow air
   // from each side.
   rad=8.5;
   backlen=10;
   backshift=12;
   frontlen=30;
   down=30;
   union() {
      difference() {
         // Subtract horn shaped bits from head to leave openings for air
         onehead(side,bladedia,down,rad);
         translate([side-backshift,down,(rad-2)+3])
            onehorn((rad/2)-1,backlen+1,frontlen,1,true);
         translate([side,0,0])
            mirror([1,0,0])
               translate([side-backshift,down,(rad-2)+3])
                   onehorn((rad/2)-1,backlen+1,frontlen,1,true);
      }
      // Subtract head shaped bits from horns to fit smooth to side
      difference() {
         translate([side-backshift,down,(rad-2)+3])
            onehorn(rad/2,backlen,frontlen);
         onehead(side,bladedia-3,down,rad-1.5,12,1,true);
      }
      translate([side,0,0])
         mirror([1,0,0])
            difference() {
               translate([side-backshift,down,(rad-2)+3])
                  onehorn(rad/2,backlen,frontlen);
               onehead(side,bladedia-3,down,rad-1.5,12,1,true);
            }
   }
}

// Just a visualization module with both the fan_trap and the shroud
// together. (Easier to put in the complete carriage visualization).
//
module fan_trap_and_shroud() {
   union() {
      fan_trap();
      translate([fanboxwall,fanboxwall,fanthick+fanboxwall])
         hornshroud();
   }
}
